/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "emea.h"

#include <QtCore/QDebug>

#include <pragreader.h>
#include <eposbin.h>
#include <evaluationpath.h>
#include <resulttospectrum.h>

class EMEAPrague : public EMEABase
{
	public:
		EMEAPrague( QObject* p ) : EMEABase( p ), strip( false ) {
			qDebug() << "EMEAPrague::EMEAPrague(" << p << ")";
			if ( QCoreApplication::arguments().contains( "-strip" ) )
				strip = true;
		}

		void setupChain( int threads ) {
			qDebug() << "EMEAPrague::setupChain(" << threads << ")";
			addToChainStart( objectInNewThread<NSDiskIO::PragDopplerReader>() );
			QList<NSCore::DataSetChainMember*> evals;
			for ( int i=0; i<threads; ++i )
				evals << new NSEvaluation::EvaluationPath( this );
			addToChain( evals );
			if ( strip ) {
				addToChain( new NSDiskIO::EPOSBinWriter( this ) );
			} else {
				NSSpectrum::ResultsToSpectrum* res = new NSSpectrum::ResultsToSpectrum( this );
				connect( res, SIGNAL( newSpectrum( NSSpectrum::AbstractSpectrumModel* ) ),
						this, SIGNAL( newSpectrum( NSSpectrum::AbstractSpectrumModel* ) ) );
                addToChain( res->chainmembers(), false );
				addToGui( res );
			}
		}

		bool strip;
};

EMEA_MAIN( EMEAPrague );

