/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include <QtCore/QDebug>
#include <QtCore/QTimer>
#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QTreeView>
#include <QtGui/QSplitter>
#include <QtGui/QPushButton>

#include "qtreeitemmodel.h"
#include "tiparameter.h"

#include "namedtreemodel_test.h"
#include "namedtreemodel_test.moc"

using namespace NSCore;


int main( int argc, char** argv )
{
	qDebug() << "Starting.";

	QApplication qapp( argc, argv );

	MyThread* thread = new MyThread();
	thread->start();

	qDebug() << " Creating model";
	QTreeItemModel* model = new QTreeItemModel( &qapp );
	thread->model = model;
	QTimer::singleShot( 10, thread, SLOT( addToModel() ) );

	qDebug() << " Creating the GUI";
	QSplitter* splitter = new QSplitter( Qt::Vertical, 0 );

	QTreeView* tv = new QTreeView( splitter );
	tv->setModel( model );
	QTimer::singleShot( 11, tv, SLOT( expandAll() ) );

	QPushButton* deleteItem = new QPushButton( "Delete one item", splitter );
	QObject::connect( deleteItem, SIGNAL( clicked() ), thread->item->getParameter( "second/first" ), SLOT( deleteLater() ) );
	QPushButton* deleteItems = new QPushButton( "Delete some items", splitter );
	QObject::connect( deleteItems, SIGNAL( clicked() ), thread->item->getParameter( "second" ), SLOT( deleteLater() ) );

	//QPushButton* reset = new QPushButton( "Reset View", splitter );
	//QObject::connect( reset, SIGNAL( clicked() ), model, SLOT( reset() ) );

	QPushButton* moreItems = new QPushButton( "More items", splitter );
	QObject::connect( moreItems, SIGNAL( clicked() ), thread, SLOT( addMore() ) );

	splitter->show();

	qDebug() << " Starting Eventloop...";
	int ret = qapp.exec();
	thread->exit();
	qDebug() << "Finished. Returncode is" << ret;
	return ret;
}
