/*
    Copyright 2009-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "qtreeitemmodelv2.h"
#include "qtreeitemmodelv2.moc"

#include "qtreeitem.h"
#include "modeltest.h"

using namespace NSCore;

class NSCore::RootItem : public NSCore::QTreeItem
{
	public:
		RootItem( QObject* p ) : QTreeItem( "RootItem", p ) {
		}

		QString debug() const { return "RootItem"; }
		QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
		bool restoreFromXML( QDomNode ) { return false; }


		struct cache_t
		{
			QPointer<QTreeItem> item;
			QModelIndex idx;
		};
		QList<cache_t> cache;
};

QTreeItemModelV2::QTreeItemModelV2( QObject* p )
	: QAbstractItemModel( p )
	, d( new RootItem( this ) )
{
	connect( d, SIGNAL( newChildAdded( NSCore::QTreeItem* ) ),
			this, SLOT( newChildAdded( NSCore::QTreeItem* ) ) );

	qDebug() << "\033[31m d =" << d << "\033[0m";

	new ModelTest( this );
}
QTreeItemModelV2::~QTreeItemModelV2() {
}

void QTreeItemModelV2::addToRootItem( NSCore::QTreeItem* item ) {
	qDebug() << "QTreeItemModelV2::addToRootItem(" << item << ")";
	//beginInsertRows( QModelIndex(), d->childs(), d->childs() );
	d->addChild( item );
	//d->debugPrint();
	//endInsertRows();
}

int QTreeItemModelV2::columnCount( const QModelIndex& ) const {
	return 2;
}
int QTreeItemModelV2::rowCount( const QModelIndex& index ) const {
	if ( !index.isValid() ) {
		//return d->childs();
		return d->childobjects<QTreeItem>().size();
	}
	if ( index.isValid() && index.column() == 0 ) {
		QTreeItem* item = static_cast<QTreeItem*>( index.internalPointer() );
		qDebug() << "\033[32mQTreeItemModelV2::rowCount(" << index << ")\033[0m internalPointer is" << item;
		if ( item != 0 ) {
			qDebug() << "  will return" << item->childs();
			//return item->childs();
			return item->childobjects<QTreeItem>().size();
		}
	}
	return 0;
}

QModelIndex QTreeItemModelV2::index( int row, int column, const QModelIndex& parent ) const {
	if ( row < 0 || column < 0 )
		return QModelIndex();
	//qDebug() << "QTreeItemModelV2::index(" << row << "," << column << "," << parent << ")";
	QTreeItem* p = static_cast<QTreeItem*>( parent.internalPointer() );
	if ( ! parent.isValid() )
		p = d;
	//qDebug() << "  internalPointer is" << p;
	//if ( row < p->childs() ) {
	if ( row < p->childobjects<QTreeItem>().size() ) {
		QModelIndex idx = createIndex( row, column, p->childobjects<QTreeItem>()[ row ] );
		//qDebug() << "  will return" << idx << " with pointer" << p->childobjects<QTreeItem>()[ row ];
		return idx;
	}
	return QModelIndex();
}
QModelIndex QTreeItemModelV2::parent( const QModelIndex& index ) const {
	if ( index.isValid() ) {
		//qDebug() << "\033[33mQTreeItemModelV2::parent(" << index << ")\033[0m" << index.internalPointer();
		QTreeItem* item = static_cast<QTreeItem*>( index.internalPointer() );
		if ( item ) {
			return indexForItem( dynamic_cast<QTreeItem*>( item->treeParent() ) );
		}
	}
	return QModelIndex();
}

QVariant QTreeItemModelV2::data( const QModelIndex& index, int role ) const {
	if ( ! index.isValid() ) {
		return QVariant();
	}
	if ( index.column() == 0 ) {
		return static_cast<QTreeItem*>( index.internalPointer() )->guiIdentifier( role );
	}
	if ( index.column() == 1 && (role == Qt::DisplayRole || role == Qt::EditRole) ) {
		return static_cast<QTreeItem*>( index.internalPointer() )->guiValue();
	}
	return QVariant();
}

bool QTreeItemModelV2::setData( const QModelIndex& index, const QVariant& value, int role ) {
	QTreeItem* item = static_cast<QTreeItem*>( index.internalPointer() );
	if ( item
			&& role == Qt::EditRole
			&& index.column() == 1
			&& item->guiValue().type() != QVariant::Invalid
			) {
		item->guiSetValue( value );
		emit dataChanged( index, index );
		return true;
	}
	return QAbstractItemModel::setData( index, value, role );
}

QVariant QTreeItemModelV2::headerData( int section, Qt::Orientation orientation, int role ) const {
	if ( orientation == Qt::Horizontal && role == Qt::DisplayRole ) {
		switch (section) {
			case 0:
				return "Name";
			case 1:
				return "Value";
		}
	}
	return QAbstractItemModel::headerData( section, orientation, role );
}

void QTreeItemModelV2::newChildAdded( NSCore::QTreeItem* newchild ) {
	QTreeItem* parent = dynamic_cast<QTreeItem*>( sender() );
	if ( parent && newchild ) {
		qDebug() << "QTreeItemModelV2::newChildAdded(" << newchild << ") parent:" << parent;
		//qDebug() << " idx=" << indexForItem( newchild );
		//qDebug() << " pidx=" << indexForItem( parent );
		int row = parent->childobjects<QTreeItem>().indexOf( newchild );
		beginInsertRows( indexForItem( parent ), row, row );

		connect( newchild, SIGNAL( newChildAdded( NSCore::QTreeItem* ) ),
				this, SLOT( newChildAdded( NSCore::QTreeItem* ) ) );

		foreach ( QTreeItem* item, newchild->childobjects<QTreeItem>() ) {
			qDebug() << " child is" << item;
			qDebug() << "  with parent" << item->parent();
			qDebug() << "  and treeParent" << item->treeParent();
		}
		endInsertRows();
	}
}



QModelIndex QTreeItemModelV2::indexForItem( QTreeItem* item ) const {
	foreach ( RootItem::cache_t c, d->cache ) {
		if ( c.item == item ) {
			return c.idx;
		}
	}
	if ( item == 0 ) return QModelIndex();
	if ( item == d ) return QModelIndex();

	//if ( item->treeParent() == 0 ) return QModelIndex();

	qDebug() << "QTreeItemModelV2::indexForItem(" << item << ")";
	qDebug() << item->parent() << "?=" << item->treeParent();

	QTreeItem* pitem = dynamic_cast<QTreeItem*>( item->treeParent() );
	if ( ! pitem ) pitem = d;
	QModelIndex pidx = indexForItem( pitem );
	qDebug() << " " << pitem->childobjects<QTreeItem>().indexOf( item );
	QModelIndex ret = index( pitem->childobjects<QTreeItem>().indexOf( item ), 0, pidx  );
	RootItem::cache_t c;
	c.item = item;
	c.idx = ret;
	d->cache.append( c );
	return ret;
}
