/*
    Copyright 2009-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "qtreeitemmodelv3.h"
#include "qtreeitemmodelv3.moc"

#include <QtCore/QDebug>

#include <qtreeitem.h>
#include <modeltest.h>

using namespace NSCore;

class NSCore::RootItem : public QTreeItem
{
	public:
		RootItem( QObject* p ) : QTreeItem( "RootItem", p ) {
		}
		QString debug() const { return "RootItem"; }
		QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
		bool restoreFromXML( QDomNode ) { return false; }

		QStringList _items;
};

QTreeItemModelV3::QTreeItemModelV3( QObject* p )
	: QAbstractItemModel( p )
	, d( new NSCore::RootItem( this ) )
{
	new ModelTest( this );
}
QTreeItemModelV3::~QTreeItemModelV3() {
}

void QTreeItemModelV3::addToRootItem( QTreeItem* item ) {
	d->addChild( item );
	addChilds( item );
}
void QTreeItemModelV3::addChilds( QTreeItem* item, QString prefix ) {
	QString name =  prefix+"/"+item->objectName();
	//d->_items.append( name );
	int idx = d->_items.indexOf( prefix );
	if ( idx == -1 ) idx = d->_items.size()-1;
	/// Begin and end for the insertRow
	d->_items.insert( idx+1, name );
	foreach ( QTreeItem* child, item->childobjects<QTreeItem>() ) {
		addChilds( child, name );
	}
}

int QTreeItemModelV3::columnCount( const QModelIndex& parent ) const {
	//qDebug() << "QTreeItemModelV3::columnCount(" << parent << ")";
	return 1;
}
int QTreeItemModelV3::rowCount( const QModelIndex& parent ) const {
	//qDebug() << "QTreeItemModelV3::rowCount(" << parent << ")";
	if ( !parent.isValid() ) {
		return d->_items.size();
	}
	return 0;
}

QModelIndex QTreeItemModelV3::index( int row, int column, const QModelIndex& parent ) const {
	//qDebug() << "QTreeItemModelV3::index(" << row << "," << column << "," << parent << ")";
	if ( !parent.isValid() && row >= 0 && row < d->_items.size() && column >= 0 && column < 1 ) {
		return createIndex( row, column );
	}
	return QModelIndex();
}
QModelIndex QTreeItemModelV3::parent( const QModelIndex& index ) const {
	//qDebug() << "QTreeItemModelV3::parent(" << index << ")";
	return QModelIndex();
}

QVariant QTreeItemModelV3::data( const QModelIndex& index, int role ) const {
	//qDebug() << "QTreeItemModelV3::data(" << index << "," << role << ")";
	if ( index.isValid() && role == Qt::DisplayRole ) {
		return d->_items.at( index.row() );
	}
	return QVariant();
}

