/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "speedproducer.h"
#include "speedproducer.moc"

#include "random.h"
#include <QtCore/QDateTime>
#include <QtCore/QTimer>
#include <QtCore/QCoreApplication>
#include <QtCore/QThread>

#include <tiaction.h>
#include <tiparameter.h>
#include <statisticslogging.h>

using namespace NSCore;


SpeedProducer::SpeedProducer( QObject* p )
    : DataSetProducer( "Speed-Producer", p )
    , _togenerate( 0 )
    //, _p( new TiParameter( "Parameters", QVariant::Invalid, this ) )
    //, _dirty( true )
{
    _ownthread = ( thread() != QCoreApplication::instance()->thread() );

    new TiAction( "Generate one event!", this, ":/icons/kde/cr22-action-player_play.png", this, SLOT( generateOneDataSet() ) );
    TiAction* action = new TiAction( "Generate N events!", this, ":/icons/kde/cr22-action-player_fwd.png", this, SLOT( generateNDataSets() ) );
    _number = new TiParameter( "Events to generate", 1, action );
}
SpeedProducer::~SpeedProducer() {
}

void SpeedProducer::start() {
    generateNDataSets();
}
void SpeedProducer::stop() {
    generateNDataSets( 0 );
}

void SpeedProducer::generateNDataSets() {
    generateNDataSets( _number->toInt() );
}
void SpeedProducer::generateNDataSets( int n ) {
    //qDebug() << "SpeedProducer::generateNDataSets(" << n << ")";
    if ( _togenerate == 0 )
        QTimer::singleShot( 100, this, SLOT( generateDataSet() ) );
    _togenerate += n;
}

QDomNode SpeedProducer::saveToXML( QDomDocument doc ) {
    QDomElement node = doc.createElement( "reader" );
    node.setAttribute( "type", "speedproducer" );
    return node;
}

bool SpeedProducer::restoreFromXML( QDomNode node ) {
    if ( node.nodeName() != "reader" || !node.isElement() || node.toElement().attribute( "type" ) != "speedproducer" )
        return false;
    return true;
}

void SpeedProducer::generateDataSet() {

    if ( QThread::currentThread() != thread() ) {
        QTimer::singleShot( 10, this, SLOT( generateDataSet() ) );
        return;
    }

    //qDebug() << "SpeedProducer::generateDataSet()";

    AcqisitionDataSet data( 1, 70 );

    double* ch = data.getRawChannel( 0 );
    for ( int i=0; i<70; i++ ) {
        ch[i] = fabs( - fabs( -10.0 + i ) + 30 ) - 30.;
    }
    data.addResult( "channel 0/fullscale", 33. );
    data.addResult( "channel 0/offset", 16. );
    data.updateMinMax();

    enqueue( data );

    --_togenerate;
    if ( _togenerate > 0 ) {
        if ( _ownthread )
            QTimer::singleShot( 0, this, SLOT( generateDataSet() ) );
        else
            QTimer::singleShot( 10, this, SLOT( generateDataSet() ) );
    } else
        emit finished();

    logging()->addStatistics( "to generate", _togenerate );
}

