/*
    Copyright 2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "statisticslogging.h"
//#include "statisticslogging.moc"

#include <QtCore/QHash>

#include <QtCore/QDebug>

namespace NSCore
{

    struct StatsLogging
    {
        QList<double> values;

        double getAvg() {
            double ret = 0.;
            if (values.size()>0) {
                foreach (double v, values) ret += v;
                ret /= values.size();
                values.clear();
            }
            return ret;
        }
    };

    struct StatisticsAndLogging::Private
    {
        QHash<QString, StatsLogging> stats;
    };

    StatisticsAndLogging::StatisticsAndLogging() : d( new Private() ) {
    }

    void StatisticsAndLogging::addStatistics(QString name, double value) {
        d->stats[name].values.append(value);
    }
    void StatisticsAndLogging::addStatistics(statisticsmap map) {
        foreach (QString name, map.keys()) {
            d->stats[name].values.append(map[name]);
        }
    }

    StatisticsAndLogging::statisticsmap StatisticsAndLogging::getStatistics() {
        StatisticsAndLogging::statisticsmap ret;
        foreach (QString name, d->stats.keys()) {
            ret[name] = d->stats[name].getAvg();
        }
        return ret;
    }

} // namespace NSCore
