
#ifndef DATATYPESTEST
#define DATATYPESTEST

#include <QtTest/QtTest>

#include <datatypes.h>

using namespace NSCore;

class DataTypesTest : public QObject
{
	Q_OBJECT
	private:
		NSCore::AcqisitionDataSet* data;
	private slots:
		void initTestCase() {
			data = new NSCore::AcqisitionDataSet( 0,0 );
		}
		void cleanupTestCase() {
		}
		void checkEmptyDataSet() {
			QVERIFY( data->samples() == 0 );
			QVERIFY( data->channels() == 0 );
			QVERIFY( data->currentGenerationNumber() == 0 );
			QCOMPARE( data->getPrefix(), QString("/") );
			QCOMPARE( data->parentGeneration(), 0 );
			QCOMPARE( data->getRawData(), data->getRawChannel( 0 ) );
			QCOMPARE( data->getRawData(), data->getRawChannel( 1 ) );
		}
		void newDataSet() {
			delete data;
			data = new NSCore::AcqisitionDataSet( 1, 100 );
			for ( int i=0; i<100; ++i )
				data->getRawChannel( 0 )[i] = 0.0;
			data->updateMinMax();
			QCOMPARE( data->channels(), 1 );
			QCOMPARE( data->samples(), 100 );
			QCOMPARE( data->currentGenerationNumber(), 0 );
			QCOMPARE( data->getResultNames().size(), 4 );
			QCOMPARE( data->getRawData(), data->getRawChannel( 0 ) );
			QCOMPARE( data->getRawData() + 100, data->getRawChannel( 1 ) );
		}
		void addResult() {
			QStringList tmp = data->getResultNames();
			data->addResult( "test", 10.001 );
			QCOMPARE( data->getResultNames().size(), 5 );
			QCOMPARE( data->getResult( "test" ), 10.001 );
			tmp << "test";
			tmp.sort();
			QStringList tmp2 = data->getResultNames();
			tmp2.sort();
			QCOMPARE( tmp2, tmp );
		}
		void secondgeneration() {
			data->newGeneration( "second", 2, 50 );
			for ( int i=0; i<100; ++i )
				data->getRawChannel( 0 )[i] = 0.0;
			data->updateMinMax();
			QCOMPARE( data->getResultNames().size(), 13 );
			QCOMPARE( data->getPrefix(), QString("/second/") );
			QCOMPARE( data->currentGenerationNumber(), 1 );
			QCOMPARE( data->parentGeneration(), 0 );
			QCOMPARE( data->channels(), 2 );
			QCOMPARE( data->samples(), 50 );
			QCOMPARE( data->getResult( "../test" ), 10.001 );
		}
		void checkthirdprefix() {
			data->pushPrefix( "third" );
			QCOMPARE( data->getPrefix(), QString( "/second/third/" ) );
			QCOMPARE( data->parentGeneration(), 1 );
			data->addResult( "bla", 3.14 );
			QCOMPARE( data->getResult( "bla" ), 3.14 );
			QCOMPARE( data->getResult( "../channel 0/min" ), 0.0 );
			QCOMPARE( data->getResult( "../../test" ), 10.001 );
			data->popPrefix( "third" );
			QCOMPARE( data->getPrefix(), QString( "/second/" ) );
		}
		void backtooriginalgeneration() {
			data->popPrefix( "second" );
			QCOMPARE( data->channels(), 1 );
			QCOMPARE( data->samples(), 100 );
			QCOMPARE( data->currentGenerationNumber(), 0 );
			QCOMPARE( data->parentGeneration(), 0 );
		}
		void benchmark_getresultnames() {
			QBENCHMARK{ data->getResultNames(); }
		}
		void benchmark_getresult_data() {
			QTest::addColumn<QString>("result");
			foreach ( QString name, data->getResultNames() ) {
				QTest::newRow(qPrintable(name)) << qPrintable(name);
			}
		}
		void benchmark_getresult() {
			QFETCH( QString, result );
			QBENCHMARK{ data->getResult( result ); }
		}
		void benchmark_samples() {
			QBENCHMARK{ data->samples(); }
		}
		void benchmark_channels() {
			QBENCHMARK{ data->channels(); }
		}
		void benchmark_currentgenerationnumber() {
			QBENCHMARK{ data->currentGenerationNumber(); }
		}
		void benchmark_updateminmax() {
			QBENCHMARK{ data->updateMinMax(); }
		}
};

#endif
