
#ifndef QTREEITEMTEST
#define QTREEITEMTEST

#include <QtTest/QtTest>
#include <cmath>

#include <tiparameter.h>

using namespace NSCore;

class NamedParameterTreeItemTest : public QObject
{
	Q_OBJECT
	private:
		TiParameter* tree;
		QString xml;
	private slots:
		void initTestCase() {
			tree = new TiParameter( "item", QVariant::Invalid, 0 );
		}
		void cleanupTestCase() {
			QPointer<TiParameter> tmp = tree;
			TiParameterPointer tmp2( tree->getParameter( "string/substring" ) );
			//tree->debugPrint();
			delete tree;
			QVERIFY( tmp == 0 );
			QVERIFY( tmp2 == 0 );
		}
		void tree_init() {
			QCOMPARE( tree->childs(), 0 );
			QCOMPARE( tree->objectName(), QString("item") );
			//tree->init();
			tree->addParameter( "integer", 1 );
			tree->addParameter( "double", 2.2 );
			tree->addParameter( "string", "bla" );
			tree->addParameter( "string/substring", "blub" );
			tree->addParameter( "boolean", QVariant( false ) );
			QCOMPARE( tree->childs(), 4 );
			QCOMPARE( tree->childlist(), QStringList() << "integer" << "double" << "string" << "boolean" );
		}
		void savetoxml() {
			QDomDocument doc( "xml-test" );
			doc.appendChild( tree->toXml( doc ) );
			xml = doc.toString();
			QVERIFY( ! xml.isEmpty() );
		}
		void additemsbetweenxml() {
			tree->getParameter( "double" )->setValue( M_PI );
			tree->getParameter( "boolean" )->setValue( true );
			tree->addParameter( "additional after xml", true );
			tree->getParameter( "string/substring" )->setValue( "foobar" );
			QCOMPARE( tree->childs(), 5 );
			delete tree->getParameter( "integer" );
			QCOMPARE( tree->childs(), 4 );
		}
		void restorefromxml() {
			QDomDocument doc( "xml-test" );
			doc.setContent( xml );
			tree->fromXml( doc.documentElement() );
			QCOMPARE( tree->childs(), 4 );
			QCOMPARE( tree->getParameter( "double" )->toDouble(), 2.2 );
			QCOMPARE( tree->getParameter( "boolean" )->toBool(), false );
			QCOMPARE( tree->getParameter( "string/substring" )->toString(), QString( "blub" ) );
		}
		void comparexml() {
			QDomDocument doc( "xml-test" );
			doc.appendChild( tree->toXml( doc ) );
			QString tmp = doc.toString();
			QVERIFY( tmp != xml );
		}
/*		void first_appendchilds() {
			NamedTreeItem* tmp = tree->child<NamedTreeItem>( "first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void first_second_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void second_first_get() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "second/first" );
			QVERIFY( ! tmp );
		}
		void first_second_first_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second/first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 3 );
		}
		void recursive_delete() {
			QPointer<NamedTreeItem> tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			delete tree->getNamedChild( "first/second" );
			QVERIFY( tmp == 0 );
		}*/
		/*void xml_finish() {
			QDomDocument doc( "bla" );
			doc.appendChild( tree->toXml( doc ) );
			qDebug() << doc.toString();
		}*/
};

#endif
