
#ifndef QTREEITEMTEST
#define QTREEITEMTEST

#include <QtTest/QtTest>

#include <qtreeitem.h>

using namespace NSCore;

class NamedTreeItem : public QTreeItem
{
	public:
		NamedTreeItem( QString n, QTreeItem* p )
			: QTreeItem( n,p )
		{
			//qDebug() << "NamedTreeItem::NamedTreeItem( QString" << n << ", QTreeItem*" << p << ")";
		}

		void init() {
			new NamedTreeItem( "first", this );
			new NamedTreeItem( "second", this );
			if ( dynamic_cast<QTreeItem*>( parent() ) )
				if ( dynamic_cast<QTreeItem*>( parent() )->objectName() == "second" )
					new NamedTreeItem( "third", this );
		}

		QDomNode saveToXML( QDomDocument doc ) {
			return doc.createElement( "namedtreeitem" );
		}
		bool restoreFromXML( QDomNode node ) {
			return node.nodeName() == "namedtreeitem";
		}

	private:
		QString debug() const {
			return objectName();
		}
};

/*class NamedPointer : public QTreeItemPointer
{
	public:
		NamedPointer( QTreeItem* o, QTreeItem* p=0 ) : QTreeItemPointer( o, p ) {
		}

	private:
		QString debug() const { return "NamedPointer"; }
};*/

class QTreeItemTest : public QObject
{
	Q_OBJECT
	private:
		NamedTreeItem* tree;
		NamedTreeItem* foreign;
	private slots:
		void initTestCase() {
			tree = new NamedTreeItem( "item", 0 );
			//tree->debugPrint();
			foreign = 0;
		}
		void cleanupTestCase() {
			QPointer<NamedTreeItem> tmp = tree;
			//tree->debugPrint();
			delete tree;
			QVERIFY( tmp == 0 );
			QVERIFY( foreign == 0 );
		}
		void tree_init() {
			QCOMPARE( tree->childs(), 0 );
			QCOMPARE( tree->objectName(), QString("item") );
			tree->init();
			//tree->debugPrint();
			QCOMPARE( tree->childs(), 2 );
		}
		void first_appendchilds() {
			NamedTreeItem* tmp = tree->child<NamedTreeItem>( "first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void first_second_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void second_first_get() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "second/first" );
			QVERIFY( ! tmp );
		}
		void first_second_first_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second/first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 3 );
		}
		void recursive_delete() {
			QPointer<NamedTreeItem> tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			delete tree->getNamedChild( "first/second" );
			QVERIFY( tmp == 0 );
			//tree->debugPrint();
		}
		void pointer() {
			QTreeItemPointer p( tree->getNamedChild( "first" ) );
			QVERIFY( p == tree->getNamedChild( "first" ) );
			QPointer<NamedTreeItem> p2( tree->getNamedChild<NamedTreeItem>( "first" ) );
			delete tree->getNamedChild( "first" );
			QVERIFY( p2 == 0 );
			QVERIFY( p == 0 );
		}
		void foreign_childs_add() {
			foreign = new NamedTreeItem( "foreign", 0 );
			tree->child( "second" )->addChild( foreign );
			//tree->debugPrint();
			QCOMPARE( tree->childs(), 1 );
			QCOMPARE( tree->child( "second" )->childs(), 1 );
		}
		void foreign_childs_delete() {
			delete foreign;
			foreign = 0;
			QCOMPARE( tree->child( "second" )->childs(), 0 );
		}
		void foreign_childs_deleteparent() {
			QVERIFY( foreign == 0 );
			tree->child( "second" )->addChild( foreign = new NamedTreeItem( "foreign2", 0 ) );
			QCOMPARE( tree->child( "second" )->childs(), 1 );
			QCOMPARE( tree->childs(), 1 );
			delete tree->child( "second" );
			QCOMPARE( tree->childs(), 0 );
			QCOMPARE( foreign->childs(), 0 );
			delete foreign;
			foreign = 0;
		}
};

#endif
