
#ifndef TIENUMTEST
#define TIENUMTEST

#include <QtTest/QtTest>
#include <cmath>

#include <tienum.h>

using namespace NSCore;

class TiEnumTest : public QObject
{
	Q_OBJECT
	private:
		TiEnum* tree;
		QString xml;
	private slots:
		void initTestCase() {
			tree = new TiEnum( "parent", QStringList() << "Bla" << "Blub", 0, 0 );
		}
		void cleanupTestCase() {
			QPointer<TiEnum> tmp = tree;
			delete tree;
			QVERIFY( tmp == 0 );
		}
		void singleitem() {
			QCOMPARE( tree->toInt(), 0 );
			QCOMPARE( tree->toString(), QString("Bla") );
			QCOMPARE( tree->toBool(), false );
			tree->setValue( 1 );
			QCOMPARE( tree->toInt(), 1 );
			QCOMPARE( tree->toString(), QString("Blub") );
			tree->setValue( QString("Bla") );
			QCOMPARE( tree->toInt(), 0 );
			QCOMPARE( tree->toString(), QString("Bla") );
		}
		void accessrobustness() {
			tree->setValue( 2 );
			QCOMPARE( tree->toInt(), 0 );
			QCOMPARE( tree->toString(), QString("Bla") );
			tree->setValue( QString("Test") );
			QCOMPARE( tree->toInt(), 0 );
			QCOMPARE( tree->toString(), QString("Bla") );
		}
		void addChilds() {
			new TiEnum( "first", QStringList() << "1" << "2" << "3", "1", tree );
			TiEnum* tmp = new TiEnum( "second", QStringList() << "true" << "false" << "maybe", 2, tree );
			new TiEnum( "subitem", QStringList() << "childs suck" << "childs rock!", 1, tmp );
			//tree->debugPrint();
			QCOMPARE( tree->childs(), 2 );
			QCOMPARE( tree->getNamedChild<TiEnum>( "second" )->childs(), 1 );
			QCOMPARE( tree->getNamedChild<TiEnum>( "second/subitem" )->childs(), 0 );
		}
		void savetoxml() {
			QDomDocument doc( "xml-test" );
			doc.appendChild( tree->toXml( doc ) );
			xml = doc.toString();
			QVERIFY( ! xml.isEmpty() );
			//qDebug() << xml;
		}
		void restorefromxml() {
			tree->getNamedChild<TiEnum>( "second" )->setValue( 0 );
			QCOMPARE( tree->getNamedChild<TiEnum>("second")->toInt(), 0 );
			QDomDocument doc( "xml-test" );
			doc.setContent( xml );
			tree->fromXml( doc.documentElement() );
			QCOMPARE( tree->getNamedChild<TiEnum>("second")->toInt(), 2 );
		}
		/*void additemsbetweenxml() {
			tree->getParameter( "double" )->setValue( M_PI );
			tree->getParameter( "boolean" )->setValue( true );
			tree->addParameter( "additional after xml", true );
			tree->getParameter( "string/substring" )->setValue( "foobar" );
			QCOMPARE( tree->childs(), 5 );
			delete tree->getParameter( "integer" );
			QCOMPARE( tree->childs(), 4 );
		}
		void restorefromxml() {
			QDomDocument doc( "xml-test" );
			doc.setContent( xml );
			tree->fromXml( doc.documentElement() );
			QCOMPARE( tree->childs(), 4 );
			QCOMPARE( tree->getParameter( "double" )->toDouble(), 2.2 );
			QCOMPARE( tree->getParameter( "boolean" )->toBool(), false );
			QCOMPARE( tree->getParameter( "string/substring" )->toString(), QString( "blub" ) );
		}
		void comparexml() {
			QDomDocument doc( "xml-test" );
			doc.appendChild( tree->toXml( doc ) );
			QString tmp = doc.toString();
			QVERIFY( tmp != xml );
		}*/
/*		void first_appendchilds() {
			NamedTreeItem* tmp = tree->child<NamedTreeItem>( "first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void first_second_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void second_first_get() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "second/first" );
			QVERIFY( ! tmp );
		}
		void first_second_first_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second/first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 3 );
		}
		void recursive_delete() {
			QPointer<NamedTreeItem> tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			delete tree->getNamedChild( "first/second" );
			QVERIFY( tmp == 0 );
		}*/
		/*void xml_finish() {
			QDomDocument doc( "bla" );
			doc.appendChild( tree->toXml( doc ) );
			qDebug() << doc.toString();
		}*/
};

#endif
