/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "tiaction.h"
#include "tiaction.moc"

#include <QtCore/QDebug>

QDataStream& operator<<( QDataStream& out, const QDomDocument& in ) {
	qDebug() << "operator<<( QDataStream, QDomDocument ) will write" << in.toString();
	out << in.toString();
	return out;
}
QDataStream& operator>>( QDataStream& in, QDomDocument& out ) {
	QString tmp;
	in >> tmp;
	out.setContent( tmp );
	return in;
}

namespace NSCore {

	TiAction::TiAction( QString n, QTreeItem* p, QString iconpath )
		: QTreeItem( n, p )
	{
		_document = QDomDocument( "epos-actions" );
		_document.appendChild( _document.createElement( "action" ) );
		_document.documentElement().setAttribute( "name", n );
	}
	TiAction::TiAction( QString n, QTreeItem* p, QObject* o, const char* s )
		:QTreeItem( n, p )
	{
		connect( this, SIGNAL( activated() ), o, s );
		_document = QDomDocument( "epos-actions" );
		_document.appendChild( _document.createElement( "action" ) );
		_document.documentElement().setAttribute( "name", n );
	}
	TiAction::TiAction( QString n, QTreeItem* p, QString i, QObject* o, const char* s )
		: QTreeItem( n, p )
	{
		connect( this, SIGNAL( activated() ), o, s );
		_document = QDomDocument( "epos-actions" );
		_document.appendChild( _document.createElement( "action" ) );
		_document.documentElement().setAttribute( "name", n );
		_document.documentElement().setAttribute( "icon", i );
	}
	TiAction::TiAction( QDomDocument xml, QTreeItem* p, QObject* o, const char* s )
		: QTreeItem( xml.documentElement().attribute( "name" ), p )
	{
		if ( xml.documentElement().nodeName() == "action" ) {
			_document = xml;
			setObjectName( _document.documentElement().attribute( "name" ) );
		}
		if ( o != 0 && s != 0 )
			connect( this, SIGNAL( activated() ), o, s );
	}

	TiAction::TiAction( TiAction& item, QTreeItem* p )
		: QTreeItem( item.objectName(), p )
	{
		qWarning() << "TiAction::TiAction(" << &item << "," << p << ") [not tested]";
		foreach( QString s, item.childlist() ) {
			TiAction* tmp = item.child<TiAction>( s );
			if ( tmp != 0 )
				new TiAction( *tmp, this );
		}
	}
	TiAction::~TiAction() {
		//qDebug() << "TiAction::~TiAction()" << path() << " " << objectName();
	}

	QString TiAction::iconPath() const { return _document.documentElement().attribute( "icon" ); }
	void TiAction::setIconPath( QString s ) { _document.documentElement().setAttribute( "icon", s ); }

	QVariant TiAction::guiValue( int ) const {
		return qVariantFromValue( _document );
	}
	void TiAction::guiSetValue( QVariant v ) {
		//qDebug() << "TiAction::guiSetValue( v.type=" << v.typeName() << ")";
		if ( v.typeName() == QByteArray( "QString" ) ) {
			//qDebug() << " string=" << v.toString();
			QDomDocument doc( "epos-actions" );
			doc.setContent( v.toString() );
			//qDebug() << " doc=" << doc.toString();
			v = qVariantFromValue<QDomDocument>( doc );
		}
		//qDebug() << " (new) type is" << v.typeName();
		if ( v.typeName() != QByteArray("QDomDocument") )
			return;
		//qDebug() << " Have a QDomDocument!";
		QDomDocument doc = qVariantValue<QDomDocument>( v );
		//qDebug() << " doc=" << doc.toString();
		emit activated( doc );
		QDomElement action = doc.documentElement();
		if ( action.nodeName() == "action" && action.isElement() ) {
			QList<QVariant> args;
			for ( QDomElement arg=action.firstChildElement( "argument" ); !arg.isNull();
					arg=arg.nextSiblingElement( "argument" ) )
			{
				QDomElement value = arg.firstChildElement( "value" );
				if (
						arg.attribute( "type" ) == "bool"
						|| arg.attribute( "type" ) == "yesno"
						|| arg.attribute( "type" ) == "okcancel"
						)
				{
					args.append( QVariant( value.text() == QString( "true" ) ) );
				}
				if ( arg.attribute( "type" ) == "int" ) {
					args.append( QVariant( value.text().toInt() ) );
				}
				if ( arg.attribute( "type" ) == "double" ) {
					args.append( QVariant( value.text().toDouble() ) );
				}
				if (
						arg.attribute( "type" ) == "string"
						|| arg.attribute( "type" ) == "listitem"
						|| arg.attribute( "type" ) == "stringlistitem"
						)
				{
					args.append( QVariant( value.text() ) );
				}
			}
			//qDebug() << " Extracted arguments are:" << args;
			if ( args.size() == 1 ) {
				if ( args[0].type() == QVariant::String )
					emit activated( args[0].toString() );
				if ( args[0].type() == QVariant::Double )
					emit activated( args[0].toDouble() );
				if ( args[0].type() == QVariant::Int )
					emit activated( args[0].toInt() );
				if ( args[0].type() == QVariant::Bool )
					emit activated( args[0].toBool() );
			}
			if ( args.size() > 0 )
				emit activated( args );
			emit activated();
		} else
			qWarning() << "TiAction::guiSetValue() something went wrong...";
	}

	Qt::ItemFlags TiAction::guiFlags() const {
		return QTreeItem::guiFlags() | Qt::ItemIsEditable | Qt::ItemIsSelectable;
	}
	QVariant TiAction::guiIdentifier( int role ) const {
		switch ( role ) {
			case Qt::ToolTipRole:
				return _tooltip;
				break;
			case Qt::DecorationRole:
				return _document.documentElement().attribute( "icon" );
				break;
			default:
				return QTreeItem::guiIdentifier( role );
		}
		return QVariant();
	}

	QDomNode TiAction::saveToXML( QDomDocument doc ) {
		QDomElement node = doc.createElement( "action" );
		node.setAttribute( "name", objectName() );
		return node;
	}
	bool TiAction::restoreFromXML( QDomNode node ) {
		if ( node.nodeName() == "action" && node.isElement() ) {
			QDomElement elem = node.toElement();
			if ( elem.attribute( "name" ) == objectName() )
				return true;
		}
		return false;
	}

};

