/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "tienum.h"
#include "tienum.moc"

#include <QtCore/QDebug>

#include "tiaction.h"

namespace NSCore {

	TiEnum::TiEnum( QString n, QStringList list, QString current, QTreeItem* p )
		: QTreeItem( n, p )
		, _list( list )
		, _current( 0 )
		, _inupdate( false )
		, _guihint( 3 )
	{
		if ( _list.contains( current ) )
			_current = _list.indexOf( current );
	}
	TiEnum::TiEnum( QString n, QStringList list, int current, QTreeItem* p )
		: QTreeItem( n, p )
		, _list( list )
		, _current( current )
		, _inupdate( false )
		, _guihint( 3 )
	{
		if ( _current-1 >= _list.size() ) _current = 0;
	}

	/*TiEnum::TiEnum( TiEnum& item, QTreeItem* p )
		: QTreeItem( item.objectName(), p )
	{
		foreach( QString s, item.childlist() ) {
			TiParameter* tmp = dynamic_cast<TiParameter*>( item.child( s ) );
			if ( tmp != 0 )
				new TiParameter( *tmp, this );
		}
	}*/
	TiEnum::~TiEnum() {
	}

	QVariant TiEnum::guiValue( int role ) const {
		if ( role == Qt::EditRole ) return _list;
		return _list[_current];
	}

	void TiEnum::setValue( QVariant v ) {
		//qDebug() << "TiEnum::setValue(" << v << ")";
		//qDebug() << " _inupdate=" << _inupdate << " canConvert()" << v.canConvert( _value.type() );

		int n = -1;
		if ( v.type() == QVariant::String )
			n = _list.indexOf( v.toString() );
		if ( v.type() == QVariant::Int )
			n = v.toInt();

		if ( !_inupdate && n > -1 && n < _list.size() ) {

			_inupdate = true; // Begin with blocking the update for other calls

			_current = n;

			emit guiValueChanged();

			_inupdate = false; // Release the lock
			//QTimer::singleShot( 20, this, SLOT( unlockUpdate() ) );
		}
	}

	void TiEnum::setValue( int n ) { setValue( QVariant( n ) ); }
	void TiEnum::setValue( bool n ) { setValue( QVariant( n ) ); }
	void TiEnum::setValue( QString n ) { setValue( QVariant( n ) ); }

	void TiEnum::reset() {
		//setValue( QVariant( _value.type() ) );
	}

	void TiEnum::unlockUpdate() {
		_inupdate = false;
	}

	void TiEnum::setGuiHint( int n ) {
		if ( _guihint & TiEnum::ProvideReset && ! n & TiEnum::ProvideReset ) {
			delete getNamedChild( "Reset" );
		}
		_guihint = n;
		if ( _guihint & TiEnum::ProvideReset ) {
			TiAction* action = new TiAction( "Reset", this );
			connect( action, SIGNAL( activated() ), this, SLOT( reset() ) );
		}
	}

	Qt::ItemFlags TiEnum::guiFlags() const {
		Qt::ItemFlags ret = QTreeItem::guiFlags()| Qt::ItemIsSelectable;
		//if ( _value.type() != QVariant::Invalid )
			ret |= Qt::ItemIsEditable ;
		return ret;
	}
	QVariant TiEnum::guiIdentifier( int role ) const {
		switch ( role ) {
			case Qt::ToolTipRole:
				return _tooltip;
				break;
			default:
				return QTreeItem::guiIdentifier( role );
		}
		return QVariant();
	}

	bool TiEnum::needsToSaveToXml() {
		if ( ! ( _guihint & WriteAllowed ) )
			return false;
		return true;
	}
	QDomNode TiEnum::saveToXML( QDomDocument doc ) {
		QDomElement node = doc.createElement( "parameter" );
		node.setAttribute( "name", objectName() );
		node.setAttribute( "type", "enum" );
		node.setAttribute( "value", toString() );
		return node;
	}
	bool TiEnum::restoreFromXML( QDomNode node ) {
		if ( node.nodeName() == "parameter" ) {
			QDomElement elem = node.toElement();
			QString name, type, value;

			name = elem.attribute( "name" );
			type = elem.attribute( "type" );
			value = elem.attribute( "value" );

			if ( name == objectName() && type == "enum" ) {
				setValue( value );
				return true;
			}
		}
		return false;
	}

};

