/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "acqirislivereader.h"

namespace NSDiskIO {

	AcqirisLiveReader::AcqirisLiveReader( QObject* p )
		: AbstractFileReader( "AcqirisLiveReader", p )
		, _samplinginterval( 0 )
		, _channelfsr( 5 )
		, _fullscale( 256 )
		, _horizontalposition( 0 )
		, _lsb( 0 )
	{
		qDebug() << "AcqirisLiveReader::AcqirisLiveReader(" << p << ")";
	}

	NSCore::AcqisitionDataSet AcqirisLiveReader::readSingleDataset() {
		if ( ! _stream.atEnd() ) {
			bool finished = false;
			int i=0;
			while ( ! _stream.atEnd() && !finished ) {
				QString line = _stream.readLine();
				if ( line[ 0 ] == '$' ) {
					QString first = line.section( " ", 0, 0 );
					QString last = line.section( " ", -1, -1 );
					if ( first == "$SAMPIVAL" )
						_samplinginterval = last.toDouble();
					if ( first == "$CHANNELFSR" )
						_channelfsr = last.toDouble();
					if ( first == "$FULLSCALE" )
						_fullscale = last.toDouble();
					if ( first == "$HORPOS" )
						_nexthorizontalposition = last.toDouble();

					if ( _horizontalposition == 0 )
						_horizontalposition = _nexthorizontalposition;

					_lsb = _channelfsr / _fullscale;

					// only stop If you have read some data:
					if ( _data.size() > 0 )
						finished = true;
				} else {
					++i;
					_data.push_back( line.toDouble() * _lsb );
				}
			}

			NSCore::AcqisitionDataSet dataset( 1, _data.size() );
			dataset.addResult( "horizontalposition", _horizontalposition );
			dataset.addResult( "samplinginterval", _samplinginterval );
			dataset.addResult( "channel 0/fullscale", _channelfsr );

			double* ch = dataset.getRawData();
			for ( int i=0; i<_data.size(); ++i ) {
				ch[ i ] = _data[ i ];
			}
			dataset.updateMinMax();

			_data.clear();
			_horizontalposition = _nexthorizontalposition;

			return dataset;
		}
		return NSCore::AcqisitionDataSet( 0,0 );
	}

}

