/*
    Copyright 2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "compactcsv.h"

#include <tiparameter.h>

#include <QtCore/QDebug>

using namespace NSDiskIO;

CompactCSVWriter::CompactCSVWriter( QObject* p ) : AbstractFileWriter( "CompactCSVWriter", p ) {
	//qDebug() << "CompactCSVWriter::CompactCSVWriter(" << p << ")";
	_config->getParameter( "File to write to" )->setUrlFilter( "Comma separated values (*.csv)\nAll files (*.*)" );
}
CompactCSVWriter::~CompactCSVWriter() {
	//qDebug() << "CompactCSVWriter::~CompactCSVWriter()";
}

void CompactCSVWriter::fileOpened( QIODevice* file ) {
	//qDebug() << "CompactCSVWriter::fileOpened(" << file << ")";
	_stream.setDevice( file );
}
void CompactCSVWriter::fileAboutToBeClosed( QIODevice* ) {
	//qDebug() << "CompactCSVWriter::fileAboutToBeClosed()";
	_stream.setDevice( 0 );
}

void CompactCSVWriter::writeSingleDataset( NSCore::AcqisitionDataSet& data ) {
	//qDebug() << "CompactCSVWriter::writeSingleDataset( data )";
	if ( _stream.device() == 0 ) return;

	bool header_changed = false;
	foreach ( QString k, data.getResultNames() ) {
		if ( !_fields.contains( k ) ) {
			_fields.append( k );
			header_changed = true;
		}
	}
	if ( header_changed ) {
		_stream << "\"_channels_\" ";
		foreach ( QString k, _fields ) _stream << "\"" << k << "\" ";
		_stream << endl;
	}

	_stream << data.channels() << " ";
	foreach ( QString k, _fields ) {
		_stream << data.getResult( k ) << " ";
	}
	_stream << endl;
}



CompactCSVReader::CompactCSVReader( QObject* p ) : AbstractFileReader( "CompactCSVReader", p ) {
	_config->getParameter( "File to read from" )->setUrlFilter( "Comma separated values (*.csv)\nAll Files (*.*)" );
}
CompactCSVReader::~CompactCSVReader() {
}

NSCore::AcqisitionDataSet CompactCSVReader::readSingleDataSet() {
	//qDebug() << "CompactCSVReader::readSingleDataSet()";
	bool header = true;
	while ( header /*&& !_stream.atEnd()*/ ) {
		header = false;
		QString line = _stream.readLine().trimmed();
		//qDebug() << line;
		if ( line.contains( '"' ) ) {
			/// Line is new heading
			//qDebug() << " line is a new heading";
			QStringList fields = line.split( "\" \"" );
			fields.replaceInStrings( "\"", "" );
			fields.pop_front();
			_fields = fields;
			header = true;
		} else if ( !line.isEmpty() ) {
			/// Line is normal data
			//qDebug() << " line is normal data";
			QStringList d = line.split( " " );
			//qDebug() << d;
			NSCore::AcqisitionDataSet data( d.first().toInt(), 0 );
			d.pop_front();
			//qDebug() << d.size() << "=?" << _fields.size();
			for ( int i=0; i<_fields.size(); ++i ) {
				data.addResult( _fields[i], d[i].toDouble() );
			}
			//data.debugPrint();
			return data;
		}
	}
	return NSCore::AcqisitionDataSet( 0,0 );
}

void CompactCSVReader::fileOpened( QIODevice* file ) {
	//qDebug() << "CompactCSVReader::fileOpened(" << file << ")";
	_stream.setDevice( file );
}
void CompactCSVReader::fileAboutToBeClosed( QIODevice* ) {
	//qDebug() << "CompactCSVReader::fileAboutToBeClosed()";
	_stream.setDevice( 0 );
}
