/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EPOSBIN_H
#define EPOSBIN_H

#include "abstractfilereader.h"
#include "abstractfilewriter.h"

namespace NSDiskIO {

	class EPOSBinWriter : public AbstractFileWriter
	{
		public:
			EPOSBinWriter( QObject* =0 );
			~EPOSBinWriter();

			void writeSingleDataset( NSCore::AcqisitionDataSet& data );

			void fileOpened( QIODevice* );
			void fileAboutToBeClosed( QIODevice* );
			void configChanged();

			QString debug() const { return "EPOSBinWriter"; }

		private:
			bool _strip;

			QDataStream _stream;
	};

	class EPOSBinReader : public AbstractFileReader
	{
		public:
			EPOSBinReader( QObject* =0 );
			~EPOSBinReader();

			//void reset();


			void fileAboutToBeClosed( QIODevice* );
			void fileOpened( QIODevice* );

			NSCore::AcqisitionDataSet readSingleDataSet();

			QString debug() const { return "EPOSBinReader"; }

		private:
			int _headertoskip;
			/**
			 * @brief version of the file to read
			 *
			 * 0 is version 1 ( without the magic-header ) and version 2 ( with the magic-header %EPOSBIN2% )
			 * 1 is version 3 ( with magic-header %EPOSBIN3% )
			 * 2 is version 4 ( with magic-header %EPOSBIN4% before each dataset. )
			 */
			int _fileversion;

			QDataStream _stream;
			QIODevice* _file;
	};

};

#endif // EPOSBIN_H

