/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "epostext.h"
#include <cmath>

#include <QtCore/QDebug>
#include <QtCore/QCoreApplication>
#include <QtCore/QThread>

namespace NSDiskIO {

	EPOSWriter::EPOSWriter( QObject* p ) : AbstractFileWriter( "EPOSWriter", p ) { }

	void EPOSWriter::fileOpened( QIODevice* f ) {
		_stream.setDevice( f );
	}
	void EPOSWriter::fileAboutToBeClosed( QIODevice* ) {
		_stream.setDevice( 0 );
	}
	void EPOSWriter::writeSingleDataset( NSCore::AcqisitionDataSet& data ) {
		if ( ! _stream.atEnd() ) {
			_stream << data.getResult( "samplinginterval" )
			         << " "
			         << data.getResult( "triggerdelay" )
			         << " "
			         << data.getResult( "horizontalposition" )
			         << " "
			         << data.getResult( "timestamp" )
			         << " "
			         << data.channels()
			         << " "
			         << data.samples()
			         << " ";
			for ( int i=0; i<data.channels(); ++i ) {
				_stream << data.getResult( QString( "channel %1/fullscale" ).arg( i ) )
				         << " "
				         << data.getResult( QString( "channel %1/offset" ).arg( i ) )
				         << " "
				         ;
				double* ch = data.getRawChannel( i );
				int s = data.samples();
				for ( int j=0; j<s; ++j )
					_stream << ch[ j ] << " ";
			}
			_stream << endl;
		}
	}

	EPOSReader::EPOSReader( QObject* p ) : AbstractFileReader( "EPOSReader", p ) { }
	void EPOSReader::fileOpened( QIODevice* f ) {
		_stream.setDevice( f );
	}
	void EPOSReader::fileAboutToBeClosed( QIODevice* ) {
		_stream.setDevice( 0 );
	}
	NSCore::AcqisitionDataSet EPOSReader::readSingleDataSet() {
		if ( ! _stream.atEnd() ) {

			double samplinginterval, triggerdelay, horizontalposition, timestamp;
			int channels, samples;
			_stream >> samplinginterval; _stream.skipWhiteSpace();
			_stream >> triggerdelay; _stream.skipWhiteSpace();
			_stream >> horizontalposition; _stream.skipWhiteSpace();
			_stream >> timestamp; _stream.skipWhiteSpace();
			_stream >> channels; _stream.skipWhiteSpace();
			_stream >> samples; _stream.skipWhiteSpace();

			NSCore::AcqisitionDataSet data( channels, samples );
			data.addResult( "samplinginterval", samplinginterval );
			data.addResult( "triggerdelay", triggerdelay );
			data.addResult( "horizontalposition", horizontalposition );
			data.addResult( "timestamp", timestamp );

			double fullscale, offset, sample;
			for ( int i=0; i<channels; ++i ) {
				_stream >> fullscale; _stream.skipWhiteSpace();
				_stream >> offset; _stream.skipWhiteSpace();
				data.addResult( QString( "channel %1/fullscale" ).arg( i ), fullscale );
				data.addResult( QString( "channel %1/offset" ).arg( i ), offset );

				double* ch = data.getRawChannel( i );
				for ( int j=0; j<samples; ++j ) {
					_stream >> sample; _stream.skipWhiteSpace();
					ch [ j ] = sample;
				}

			}
			data.updateMinMax();

			return data;
		}
		return NSCore::AcqisitionDataSet( 0,0 );
	}

}

