/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PRAGREADER_H
#define PRAGREADER_H

#include "abstractfilereader.h"

namespace NSDiskIO {

    /**
     * @brief Reader for the binary files from prague
     */
    class PragReader : public AbstractFileReader
    {
        Q_OBJECT
        public:
            PragReader( QObject* =0 );
            ~PragReader();

            NSCore::AcqisitionDataSet readSingleDataSet();

            void fileOpened( QIODevice* );
            void fileAboutToBeClosed( QIODevice* );

            QString debug() const { return "PragReader"; }

        private slots:
            void configChanged();

        private:
            QDataStream _stream;

            QVector<double> tdata;

            int _samples;
            int _channels;
            int _offset;
    };

    class PragDopplerReader : public AbstractFileReader
    {
        public:
            PragDopplerReader( QObject* =0 );
            ~PragDopplerReader();

        private:
            QString debug() const { return "PragDopplerReader"; }

            void fileAboutToBeClosed( QIODevice* );
            void fileOpened( QIODevice* );

            NSCore::AcqisitionDataSet readSingleDataSet();

            qint32 _samples;
            qint32 _channels;
            qint32 _current_idx;

            QDataStream _stream;
    };

};

#endif // PRAGREADER_H

