/**
 *
 * Data format for the plugins-descriptions:
 *
 * /plugins : stringlist of all plugins ( could be done by top-level-groups? )
 *
 * /Plugin/ : All stuff of one plugin, valid subkeys are:
 * /Plugin/type : one of Analyzer,Modifier,Filter
 * /Plugin/categories : stringlist of categories for this plugin
 * /Plugin/results : stringlist of results this plugin may add the the result-tree.
 * /Plugin/help : Help-string to be displayed in the gui describing the plugin.
 *
 */
#include <QtCore>

struct plugin_desc_t {
	QString name;
	QString header;
	QString type;
	QString help;
	QStringList categories;
	QStringList results;
};

char* file_template = ( char* )"\
/** \n\
 * This file was generated by 'pluginfactorycreator'.\n\
 * Don't edit it by hand or submit it to the version control system.\n\
 * It will be re-generated the next time the build system runs...\n\
 */\n\
#include <plugins_interface.h>\n\
$INCLUDES\n\
\n\
class $NAME_Client : public NSEvaluation::PluginFactory_Client {\n\
	public:\n\
		NSEvaluation::Plugin* create( QString s, NSCore::QTreeItem* p ) {\n\
$CREATE\
			return 0;\n\
		}\n\
		QString type( QString s ) {\n\
$TYPE\
			return \"\";\n\
		}\n\
		QStringList canCreate() {\n\
			return QStringList()$CANCREATE;\n\
		}\n\
		QStringList categories( QString s ) {\n\
$CATEGORIES\
			return QStringList();\n\
		}\n\
		QString help( QString s ) {\n\
$HELP\
			return NSEvaluation::PluginFactory_Client::help( s );\n\
		}\n\
};\n\
\n\
extern \"C\" {\n\
	void $NAME_start() {\n\
		new $NAME_Client();\n\
	}\n\
}\n\
";
char* create_template = ( char* )"			if( s == \"$PLUGIN\" )\n\
				return new $PLUGIN( p );\n";
char* type_template = ( char* )"			if( s == \"$PLUGIN\" )\n\
				return \"$TYPE\";\n";
char* categories_template = ( char* )"			if( s == \"$PLUGIN\" )\n\
				return QStringList()$CATEGORIES;\n";
char* help_template = ( char* )"			if( s == \"$PLUGIN\" )\n\
				return \"$HELP\";\n";

int main( int argc, char** argv )
{
	QCoreApplication app( argc, argv );

	QStringList args = app.arguments();
	args.pop_front();
	foreach( QString file, args ) {
		if ( QFile::exists( file ) ) {
			//qDebug() << "Found valid file:" << file;
			QSettings settings( file, QSettings::IniFormat );

			QList<plugin_desc_t> plugins;

			QString defaultheader = settings.value( "header" ).toString();
			QString defaulttype = settings.value( "type" ).toString();
			QStringList defaultcategories = settings.value( "categories" ).toStringList();

			foreach( QString plugin, settings.childGroups() ) {
				//qDebug() << "Plugin =" << plugin;
				settings.beginGroup( plugin );
				plugin_desc_t plug;
				plug.name = plugin;
				plug.header = settings.value( "header", defaultheader ).toString();
				plug.type = settings.value( "type", defaulttype ).toString();
				plug.help = settings.value( "help" ).toString().replace("\n","\\n");
				plug.results = settings.value( "results" ).toStringList();
				plug.categories = settings.value( "categories", defaultcategories ).toStringList();
				plugins.append( plug );
				settings.endGroup();
			}

			QFile f( file.section( ".", 0, 0 )+( ".cpp" ) );
			if ( f.open( QIODevice::WriteOnly ) ) {
				//qDebug() << "writing file" << f.fileName();

				QString cpp( file_template );

				cpp.replace( "$NAME", f.fileName().section( "/", -1 ).section( ".", 0, 0 ) );

				QString create;
				QStringList includes;
				QString type;
				QString help;
				QString canCreate;
				QString categories;
				foreach( plugin_desc_t plugin, plugins ) {
					QString tmp( create_template );
					tmp.replace( "$PLUGIN", plugin.name );
					create += tmp;
					tmp = type_template;
					tmp.replace( "$PLUGIN", plugin.name );
					tmp.replace( "$TYPE", plugin.type );
					type += tmp;
					tmp = help_template;
					tmp.replace( "$PLUGIN", plugin.name );
					tmp.replace( "$HELP", plugin.help );
					help += tmp;
					canCreate+= " << \"" + plugin.name +"\"";
					if ( ! includes.contains( plugin.header ) )
						includes.append( plugin.header );
					QString cats;
					foreach( QString cat, plugin.categories )
						cats += " << \"" + cat + "\"";
					tmp = categories_template;
					tmp.replace( "$CATEGORIES", cats );
					tmp.replace( "$PLUGIN", plugin.name );
					categories += tmp;
				}
				cpp.replace( "$CREATE", create );
				cpp.replace( "$TYPE", type );
				cpp.replace( "$HELP", help );
				cpp.replace( "$CANCREATE", canCreate );
				cpp.replace( "$CATEGORIES", categories );

				QString include;
				foreach( QString inc, includes ) {
					include += "#include \"" + inc + "\"\n";
				}
				cpp.replace( "$INCLUDES", include );

				QTextStream stream( &f );

				stream << cpp;

				f.close();
			}
		}
	}
}

