/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "fancylistview.h"
#include "fancylistview.moc"
#include "fancylistdialog.h"

#include <QtCore/QDebug>
#include <QtCore/QVariant>

Q_DECLARE_METATYPE( FancyListDialog::ListItem )

#include <QtGui/QWidget>

#include <QtGui/QMessageBox>
#include <QtGui/QInputDialog>
#include <QtGui/QHeaderView>
#include <QtGui/QTreeWidgetItem>
#include <QtGui/QPushButton>
#include <QtGui/QPainter>

#include <QtGui/QTextDocument>
#include <QtGui/QAbstractTextDocumentLayout>

#include "collapsiblewidget.h"


FancyListDialog::FancyListDialog( const QString& title, const QString& desc, QWidget* p ) : QDialog( p )
{
	qDebug() << "FancyListDialog::FancyListDialog(" << title << "," << desc << "," << p << ")";
	d = new Ui_FancyListDialog;
	d->setupUi( this );
	d->label->setText( desc );
	setWindowTitle( title );

	d->buttonBox->button( QDialogButtonBox::Ok )->setEnabled( false );

	_model = new QStandardItemModel( 0, 1, this );
	d->treeView->setModel( _model );
	d->treeView->header()->hide();
	connect(
			d->treeView->selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
			this, SLOT( selectionChanged( const QItemSelection&, const QItemSelection& ) )
		);

	d->search->setFocus( Qt::OtherFocusReason );
}
FancyListDialog::~FancyListDialog() {
	delete d;
}
void FancyListDialog::selectionChanged( const QItemSelection& _current, const QItemSelection& /*_previous*/ ) {
	//qDebug() << "FancyListDialog::currentChanged(" << _current.size() << "," << _previous.size() << ")";
	QStandardItem* current = _model->itemFromIndex( _current.indexes().first() );
	//qDebug() << " current=" << current->text();

	// Category handling
	if ( _categories.contains( current ) ) {
		d->treeView->collapseAll();
		d->treeView->expand( _current.indexes().first() );
	}
	// Item handling
	d->buttonBox->button( QDialogButtonBox::Ok )->setEnabled( _items.contains( current ) );
	if ( _items.contains( current ) ) {
		currentItem = current->text();
	} else
		currentItem = "";
}
void FancyListDialog::on_search_textEdited( const QString& search ) {
	//qDebug() << "FancyListDialog::on_search_textEdited(" << search << ")";
	QList<QStandardItem*> found = _model->findItems( search, Qt::MatchContains | Qt::MatchWrap | Qt::MatchRecursive );
	if ( found.size() > 0 )
		d->treeView->setCurrentIndex( _model->indexFromItem( found.first() ) );
}
void FancyListDialog::addItems( QList<FancyListDialog::ListItem> items ) {
	//qDebug() << "FancyListDialog::addItems( <List of" << items.size() << "items> )";
	foreach( FancyListDialog::ListItem item, items ) {
		if ( item.categories.size() == 0 )
			item.categories.append( "No category" );
		foreach( QString cat, item.categories ) {
			QStandardItem* treeitem = new QStandardItem( item.title );
			treeitem->setToolTip( item.help );
			_items.append( treeitem );
			QStandardItem* catitem = 0;
			foreach( QStandardItem* tmp, _categories )
				if ( tmp->text() == cat )
					catitem = tmp;
			if ( catitem == 0 ) {
				QStandardItem* top = new QStandardItem( cat );
				top->setBackground( palette().alternateBase() );
				_model->appendRow( top );
				_categories.append( top );
				catitem = top;
			}
			catitem->appendRow( treeitem );
		}
	}
}
void FancyListDialog::addXmlItem( QDomElement elem ) {
	QStringList categories;
	QDomElement cat = elem.firstChildElement( "categories" ).firstChildElement( "category" );
	while ( ! cat.isNull() ) {
		categories << cat.firstChild().nodeValue();
		cat = cat.nextSiblingElement( "category" );
	}
	if ( categories.isEmpty() ) categories << "No category";
	foreach ( QString cat, categories ) {
		QStandardItem* treeitem = new QStandardItem( elem.firstChildElement( "title"
					).firstChild().nodeValue() );
		treeitem->setToolTip( elem.firstChildElement( "help" ).firstChild().nodeValue() );
		_items.append( treeitem );
		QStandardItem* catitem = 0;
		foreach( QStandardItem* tmp, _categories )
			if ( tmp->text() == cat )
				catitem = tmp;
		if ( catitem == 0 ) {
			QStandardItem* top = new QStandardItem( cat );
			top->setBackground( palette().alternateBase() );
			_model->appendRow( top );
			_categories.append( top );
			catitem = top;
		}
		catitem->appendRow( treeitem );
	}
}

