/*
    Copyright 2005-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef OSCIWIDGET_V3_H
#define OSCIWIDGET_V3_H

#include <QtGui/QFrame>

#include <datasetchain.h>

class QVBoxLayout;
class QTimer;

class OsciWidgetV3_Options;
class OsciWidgetV3_Osci;
class OsciWidgetV3_ChainMember;


/**
 * @brief widget for displaying NSCore::AcqisitionDataSet
 *
 * This provides a view like an oscilloscope to watch the datasets. There is a
 * number of ( currently hardcoded ) results that are shown too. It allows for
 * any number of data-widgets to be shown, where for each data-widget the
 * channel and generation can be selected freely.
 */
class OsciWidgetV3 : public QWidget
{
	Q_OBJECT
	public:
		/**
		 * @brief Construct a new OsciWidget
		 *
		 * The widget shows as man channels as selected.
		 *
		 * To plug it into the evaluation-chain, get the DataSetChainMember from
		 * chainMember().
		 */
		OsciWidgetV3( QWidget* );
		/// standard destructor
		~OsciWidgetV3();

		/**
		 * @brief Access the real ChainMember
		 *
		 * This returns a pointer to the real chain-member to plug it into the
		 * chain. This might even run in its own thread in the future.
		 */
		NSCore::DataSetChainMember* chainMember();

	private slots:
		/// Change the number of displayed channels.
		void setChannels( int );

		/// Force the usage of Min/Max for display in all channels
		void forceMinMaxUsage( bool );

		/// New data has arrived, process it.
		void dataAvailable( NSCore::AcqisitionDataSet );

	private:
		/**
		 * @{
		 *
		 * @internal
		 */
		QVBoxLayout* _layout;
		OsciWidgetV3_Options* _options;
		QList <OsciWidgetV3_Osci*> _oscis;
		OsciWidgetV3_ChainMember* _chainmember;
		// @}
};

#endif // OSCIWIDGET_V3_H

