/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef OSCIWIDGET_V3_PRIVATE_H
#define OSCIWIDGET_V3_PRIVATE_H

#include <QtGui/QFrame>
#include "osciwidget_v3_options_gui.h"
#include <QtCore/QMap>
#include <QtCore/QTime>

namespace NSCore {
	class AcqisitionDataSet;
};

class QMutex;

/**
 * @internal
 */
class OsciWidgetV3_ChainMember : public NSCore::DataSetChainMember
{
	Q_OBJECT
	/// Only OsciWidgetV3 can do anything with this.
	friend class OsciWidgetV3;
	protected:
		OsciWidgetV3_ChainMember( QObject* );

		QString debug() const { return "OsciWidgetV3_ChainMember"; }

		QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
		bool restoreFromXML( QDomNode ) { return false; }

		bool dataArrived( NSCore::AcqisitionDataSet );

	signals:
		void data( NSCore::AcqisitionDataSet );

	private:
		QTime _slowdown;
};

/**
 * @internal
 */
class OsciWidgetV3_Osci : public QWidget
{
	Q_OBJECT
	/// Only OsciWidgetV3 can do anything with this.
	friend class OsciWidgetV3;
	protected:
		OsciWidgetV3_Osci( QWidget* );
		~OsciWidgetV3_Osci();

		void data( NSCore::AcqisitionDataSet );

		void paintEvent( QPaintEvent* );

		void setChannel( int );

		void contextMenuEvent( QContextMenuEvent* );

	protected slots:
		void enterChannel();
		void enterGeneration();

		void uncheckAllResults();

		void forceMinMaxUsage( bool );

	private:
		QList<double> _data;
		double fullscale, offset;
		int _channel, _generation;
		QMutex* _mutex;
		QMap<QString,double> _addons_x;
		QMap<QString,double> _addons_y;
		QMenu* x_actions;
		QMenu* y_actions;
		QMap<QString,QAction*> _actions_x;
		QMap<QString,QAction*> _actions_y;
		bool _forceminmax;
};

/**
 * @internal
 */
class OsciWidgetV3_Options : public QFrame, public Ui::OsciWidgetV3_Options_Gui
{
	Q_OBJECT
	/// Only OsciWidgetV3 can do anything with this.
	friend class OsciWidgetV3;
	protected:
		OsciWidgetV3_Options( QWidget* );
};

#endif // OSCIWIDGET_V3_PRIVATE_H

