/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "resultviewer.h"
#include "resultviewer.moc"
#include "resultviewer_p.h"

#include <QtCore/QDebug>
#include <QtGui/QVBoxLayout>

class ResultViewer::ResultViewerPrivate : public NSCore::DataSetChainMember {
public:
	ResultViewerPrivate( QTableWidget* w, QObject* parent )
		: NSCore::DataSetChainMember( "ResultViewerConsumer", parent )
		, pw( w )
	{
		setObjectName( "Result Viewer" );
		w->setObjectName( "Result Viewer" );
		//qDebug() << "ResultViewerPrivate::ResultViewerPrivate() pw =" << pw << " pw->layout =" << pw->layout();
		_slowdown.start();
	}

	QString debug() const { return "ResultViewer"; }
	QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
	bool restoreFromXML( QDomNode ) { return false; }

	bool dataArrived( NSCore::AcqisitionDataSet data ) {
		if ( _slowdown.elapsed() > 200 ) {
			QMap<QString,double> results = data.getResultMap();
			foreach ( QString k, results.keys() ) {
				if ( !widgets.contains( k ) ) {
					ResultWidget* tmp = new ResultWidget( k, pw );
					widgets.insert( k, tmp );
				}
				widgets[ k ]->addValue( results[ k ] );
			}
			_slowdown.restart();
		}
		return true;
	}

	QHash<QString,ResultWidget*> widgets;

	QTableWidget* pw;
	QTime _slowdown;
};

ResultViewer::ResultViewer( QWidget* p )
	: QTableWidget( p )
	, d( new ResultViewerPrivate( this, this ) )
{
	setColumnCount( 5 );
	setHorizontalHeaderLabels( QStringList() << "Name" << "Minimum" << "Last" << "Average" << "Maximum" );
}
ResultViewer::~ResultViewer() {
}

NSCore::DataSetChainMember* ResultViewer::chainMember() {
	return d;
}

