/*
    Copyright 2009-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NETCLIENT_H
#define NETCLIENT_H

#include <QtCore/QObject>
#include <QtCore/QMap>

#include <qtreeitem.h>

class QIODevice;

namespace NSNetwork
{

	class AbstractNetClient: public QObject
	{
		Q_OBJECT
		public:
			AbstractNetClient( QIODevice*, QObject* =0 );
			~AbstractNetClient();

			QStringList allvariables();
			QString gettype( const QString& name );
			QString getvariable( const QString& name );
			QString setvariable( const QString& name, const QString& value );

			void subscribe( const QString& name );
			void unsubscribe( const QString& name );

		protected:
			virtual void updateVariable( const QString& name, const QString& value ) =0;

		protected slots:
			void readData();

		protected:
			QIODevice* device;
			QMap<QString,QString> values;
			volatile mutable bool _blockupdates;
			QStringList answers;
	};

	class QTreeItemNetClient: public AbstractNetClient
	{
		Q_OBJECT
		public:
			QTreeItemNetClient( QIODevice*, QObject* =0 );
			~QTreeItemNetClient();

			NSCore::QTreeItem* tree;

		private slots:
			void valueChanged();

			void actionExecuted( QDomDocument );

			void updateTree( const QStringList& );

		signals:
			void internalUpdateTree( const QStringList& );

		private:
			void updateVariable( const QString&, const QString& );

			/**
			  @internal
			  used by updateTree only
			  */
			QStringList _oldremotepaths;
	};

} // namespace NSNetwork
#endif // NETCLIENT_H
