/*
    Copyright 2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/
#ifndef NETGAUSS_H
#define NETGAUSS_H

#include <QtGui/QSplitter>
#include <QtCore/QTimer>

#include "networkchain.h"
#include <osciwidget_v3.h>
#include <gaussproducer.h>
#include <qtreeitemmodel.h>
#include <qtreeitemmodeldelegate.h>
#include "netserver.h"


class NetGauss: public QSplitter
{
    Q_OBJECT
    public:
        NetGauss(QWidget* p=0) {
            model = new NSCore::QTreeItemModel( this );

            gauss = new NSCore::GaussProducer( this );
            model->addToRootItem( gauss );

            server = new NSNetwork::NetServer( model->rootItem(), this );
            connect( server, SIGNAL( setStatus(QString) ), this, SLOT( setStatus(QString) ) );

            sender = new NSNetwork::NetworkDataSender( this );
            model->addToRootItem( sender );

            QTreeView* tv = new QTreeView( this );
            tv->setItemDelegate( new QTreeItemModelDelegate( tv ) );
            tv->setModel( model );
            OsciWidgetV3* osci = new OsciWidgetV3( this );

            gauss->registerConsumer( sender );
            gauss->registerConsumer( osci->chainMember() );

            QTimer* timer = new QTimer( this );
            connect( timer, SIGNAL( timeout() ), this, SLOT( checkStatus() ) );
            timer->start( 500 );
        }

    private slots:
        void setStatus(QString status) {
            if ( status.toLower() == "start" ) gauss->start();
            if ( status.toLower() == "stop" ) gauss->stop();
            if ( status.toLower() == "abort" ) gauss->abort();
            if ( status.toLower() == "reset" ) gauss->reset();

            checkStatus();
        }
        void checkStatus() {
            server->updateStatus( (gauss->isRunning())?"Running":"Idle" );
        }

    private:
        NSCore::QTreeItemModel* model;
        NSCore::GaussProducer* gauss;
        NSNetwork::NetServer* server;
        NSNetwork::NetworkDataSender* sender;
        OsciWidgetV3* osci;
        QTreeView* tv;
};
#endif // NETGAUSS_H
