/*
    Copyright 2009-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NETPROTOCOL_H
#define NETPROTOCOL_H

#include <QtCore/QObject>
#include <QtCore/QStringList>
#include <QtCore/QHash>

class QIODevice;

namespace NSCore {
    class QTreeItem;
};

namespace NSNetwork
{

    class AbstractNetProtocol: public QObject
    {
        Q_OBJECT
        public:
            AbstractNetProtocol( QIODevice*, QObject* =0 );

        public slots:
            void updateVariable( const QString&, const QString& );

            void updateVariablesList();

            void updateStatus( const QString& );

            void addToSpectrumList( const QString& );
            void removeFromSpectrumList( const QString& );

        signals:
            void setStatus(QString newstatus);

            void saveXml( QString& );
            void loadXml( QString );

            void getSpectrum( QString name, QByteArray& data );

        protected:
            virtual QStringList getvariables() =0;
            virtual QString getvariable( const QString& ) =0;
            virtual QString gettype( const QString& ) =0;
            virtual void setvariable( const QString&, const QString& ) =0;

            virtual QString getxml();
            virtual void setxml( QString xml );

        protected slots:
            void canReadData();

            void sendUpdate( const QString&, const QString& );
            void sendUpdate( const QString&, const QStringList& );

        protected:

            void timerEvent( QTimerEvent* );

            QIODevice* device;

            QHash<QString, QString> subscriptions;

            QString _status;

            QStringList _spectra;

        private:
            bool _updateSent;
    };

    class QTreeItemNetProtocol : public AbstractNetProtocol
    {
        Q_OBJECT
        public:
            QTreeItemNetProtocol( NSCore::QTreeItem* t, QIODevice* );
            ~QTreeItemNetProtocol();

        protected:
            QStringList getvariables();
            QString gettype( const QString& );
            QString getvariable( const QString& );
            void setvariable( const QString&, const QString& );

        private slots:
            void valueChanged();

            //void childsChanged();
            void connectNewChilds( NSCore::QTreeItem* );

        private:

            QStringList recursiveVariableNames( NSCore::QTreeItem* );

            NSCore::QTreeItem* tree;

            NSCore::QTreeItem* getItem( const QString& );

            bool _blockupdates;

            Q_DISABLE_COPY( QTreeItemNetProtocol );
    };

} // namespace NSNetwork

#endif // NETPROTOCOL_H
