/*
    Copyright 2009-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "netserver.h"
#include "netserver.moc"

#include <QtCore/QDebug>
#include <QtNetwork/QTcpServer>
#include <QtNetwork/QTcpSocket>

#include "bonjourserviceregister.h"

using namespace NSNetwork;

NetServer::NetServer( NSCore::QTreeItem* t, QObject* p )
    : QObject( p )
    , tcpserver( new QTcpServer( this ) )
    , tree( t )
{
    connect( this, SIGNAL(addToSpectrumList(const QString&)),
            this, SLOT(onAddToSpectrumList(const QString&)));
    connect( tcpserver, SIGNAL( newConnection() ),
            this, SLOT( on_tcpserver_newConnection() ) );
    /*
     * TODO: switch to automatic ports
     * Can only happen when the windows clients also use zeroconf to determine the ports.
     */
    int port = 5283;
    if ( ! tcpserver->listen( QHostAddress::Any, port ) ) {
        tcpserver->listen( QHostAddress::Any, 0 );
        port = tcpserver->serverPort();
    }
    service = new BonjourServiceRegister( this );
    BonjourRecord record( QString("EPOS client @%1:%2").arg(QHostInfo::localHostName()).arg(port), "_x-eposctrl._tcp", "" );
    service->registerService( record, port );
}
NetServer::~NetServer() {
}

void NetServer::on_tcpserver_newConnection() {
    qDebug() << "NetServer::on_tcpserver_newConnection()";
    while ( tcpserver->hasPendingConnections() ) {
        QTcpSocket* tmp = tcpserver->nextPendingConnection();
        connect( tmp, SIGNAL( disconnected() ), tmp, SLOT( deleteLater() ) );

        QTreeItemNetProtocol* proto = new QTreeItemNetProtocol( tree, tmp );

        connect( proto, SIGNAL( setStatus(QString) ),
                this, SIGNAL( setStatus(QString) ) );
        connect( proto, SIGNAL( saveXml(QString&) ),
                this, SIGNAL( saveXml(QString&) ) );
        connect( proto, SIGNAL( loadXml(QString) ),
                this, SIGNAL( loadXml(QString) ) );
        connect( this, SIGNAL( internalUpdateStatus(QString) ),
                proto, SLOT( updateStatus(QString) ) );
        connect( this, SIGNAL( addToSpectrumList( const QString& ) ),
                proto, SLOT( addToSpectrumList( const QString& ) ) );
        connect( this, SIGNAL( removeFromSpectrumList( const QString& ) ),
                proto, SLOT( removeFromSpectrumList( const QString& ) ) );
        connect( this, SIGNAL( addToSpectrumList( const QString& ) ),
                this, SLOT( onAddToSpectrumList( const QString& ) ) );
        connect( this, SIGNAL( removeFromSpectrumList( const QString& ) ),
                this, SLOT( onRemoveFromSpectrumList( const QString& ) ) );
        connect( proto, SIGNAL( getSpectrum( QString, QByteArray& ) ),
                this, SIGNAL( getSpectrum( QString, QByteArray& ) ) );

        foreach (QString spec, _cache_spectra)
            proto->addToSpectrumList(spec);
    }
}

void NetServer::updateStatus( const QString& status ) {
    emit internalUpdateStatus(status);
}
void NetServer::onAddToSpectrumList( const QString& spec ) {
    if ( ! _cache_spectra.contains( spec ) )
        _cache_spectra.append(spec);
}
void NetServer::onRemoveFromSpectrumList( const QString& spec ) {
    _cache_spectra.removeAll( spec );
}

