/*
    Copyright 2009-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NETSERVER_H
#define NETSERVER_H

#include <QtCore/QObject>
#include <qtreeitem.h>
#include "netprotocol.h"

class QTcpServer;

namespace NSNetwork
{

    class BonjourServiceRegister;

    class NetServer : public QObject
    {
        Q_OBJECT
        public:
            NetServer( NSCore::QTreeItem*, QObject* =0 );
            ~NetServer();

        public slots:
            void updateStatus( const QString& );

#define protected public
        signals:
            void addToSpectrumList( const QString& );
            void removeFromSpectrumList( const QString& );

        signals:
            void setStatus( QString newstatus );

            void saveXml( QString& );
            void loadXml( QString );

            void getSpectrum( QString name, QByteArray& data );

            void internalUpdateStatus( QString );

        private slots:
            void on_tcpserver_newConnection();
            void onAddToSpectrumList( const QString& );
            void onRemoveFromSpectrumList( const QString& );

        private:
            QTcpServer* tcpserver;
            QPointer<NSCore::QTreeItem> tree;
            BonjourServiceRegister* service;
            QStringList _cache_spectra;
    };

} // namespace NSNetwork

#endif // NETSERVER_H
