/*
    Copyright 2009-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "serversearch.h"
#include "serversearch.moc"

#include <QtNetwork/QTcpSocket>
#include "netclient.h"
#include "bonjourservicebrowser.h"
#include "bonjourserviceresolver.h"

using namespace NSNetwork;

ServerSearch::ServerSearch( QObject* p ) : QObject( p ) {
	BonjourServiceBrowser* browser = new BonjourServiceBrowser( this );
	connect( browser, SIGNAL( currentBonjourRecordsChanged( const QList<BonjourRecord>& ) ),
			this, SLOT( currentRecordsChanged( const QList<BonjourRecord>& ) ) );
	BonjourServiceResolver* resolver = new BonjourServiceResolver( this );
	connect( resolver, SIGNAL( bonjourRecordResolved( BonjourRecord ) ),
			this, SLOT( recordResolved( BonjourRecord ) ) );
	connect( this, SIGNAL( resolveRecord( BonjourRecord ) ),
			resolver, SLOT( resolveBonjourRecord(const BonjourRecord & ) ) );
	browser->browseForServiceType( "_x-eposctrl._tcp" );
}
ServerSearch::~ServerSearch() {
}

void ServerSearch::currentRecordsChanged( const QList<BonjourRecord>& records ) {
	foreach ( BonjourRecord rec, records )
		emit resolveRecord( rec );
}
void ServerSearch::recordResolved( BonjourRecord record ) {
	//qDebug() << "ServerSearch::recordResolved(" << record.host.hostName() << "," << record.port << ")";
	foreach (QAbstractSocket* socket, _sockets) {
		if ( record.host.addresses().contains( socket->peerAddress() ) && record.port == socket->peerPort() )
			return;
	}
	QTcpSocket* socket = new QTcpSocket( this );
	connect( socket, SIGNAL( disconnected() ), this, SLOT( deleteSocket() ) );
	socket->connectToHost( record.host.hostName(), record.port );
	if ( socket->waitForConnected() ) {
		NSNetwork::QTreeItemNetClient* client = new NSNetwork::QTreeItemNetClient( socket, socket );
		client->tree->setObjectName( QString( "Remote: %1:%2" ).arg( record.host.hostName() ).arg( record.port ) );
		_sockets.append( socket );
		emit newClient( client->tree );
	}
}

void ServerSearch::deleteSocket() {
	QAbstractSocket* socket = dynamic_cast<QAbstractSocket*>( sender() );
	if ( socket ) {
		_sockets.removeAll( socket );
		socket->deleteLater();
	}
}


