
#ifndef LANGUAGESCANNERTEST
#define LANGUAGESCANNERTEST

#include <QtTest/QtTest>

#include <netclient.h>
#include <QtCore>

#include "iobuffer.h"
#include "iobuffer.moc"
#include "testnetprotocol.h"
#include "testnetprotocol.moc"

using namespace NSNetwork;

class MyClient: public AbstractNetClient
{
	public:
		MyClient( QIODevice* dev, QObject* p=0 ) : AbstractNetClient( dev, p ) {
		}

		QMap<QString,QString> values;

		void updateVariable( const QString& name, const QString& value ) {
			//qDebug() << "MyClient::updateVariable(" << name << "," << value << ")";
			values[ name ] = value;
		}
};

class NetClientTest : public QObject
{
	Q_OBJECT
	public:
		NetClientTest() : client( 0 ), protocol( 0 ), pipe( 0 ) {
		}
	private:
		MyClient* client;
		TestNetProtocol* protocol;
		DebuggingPipe* pipe;
	private slots:
		void initTestCase() {
			pipe = new DebuggingPipe( this );
			QVERIFY( pipe->first()->write( "Bla" ) == 3 );
			QVERIFY( pipe->second()->write( "Blub" ) == 4 );
			QVERIFY( pipe->second()->readAll() == "Bla" );
			QVERIFY( pipe->first()->readAll() == "Blub" );
			protocol = new TestNetProtocol( pipe->first(), this );
			client = new MyClient( pipe->second(), this );
		}
		void cleanupTestCase() {
			if ( client )
				delete client;
			if ( protocol )
				delete protocol;
			if ( pipe )
				delete pipe;
		}
		void cleanup() {
			QVERIFY( pipe->first()->bytesAvailable() == 0 );
			QVERIFY( pipe->second()->bytesAvailable() == 0 );
		}
		void getvariables() {
			QStringList tmp = client->allvariables();
			QCOMPARE( tmp, QStringList() << "variable1" << "path to/variable 2" );
		}
		void getvar1() {
			QString tmp = client->getvariable( "variable1" );
			QCOMPARE( tmp, QString( "0.0815" ) );
		}
		void setvar2() {
			QString t1 = client->getvariable( "path to/variable 2" );
			QCOMPARE( t1, QString( "4711" ) );
			QString t2 = client->setvariable( "path to/variable 2", "42" );
			QCOMPARE( t2, QString( "42" ) );
		}
		void gettypes_data() {
			QTest::addColumn<QString>( "var" );
			QTest::addColumn<QString>( "type" );

			QTest::newRow( "var1" ) << "variable1" << "double";
			QTest::newRow( "var2" ) << "path to/variable 2" << "int";
			QTest::newRow( "invalid" ) << "invalid path" << "invalid";
		}
		void gettypes() {
			QFETCH( QString, var );
			QFETCH( QString, type );
			QCOMPARE( client->gettype( var ), type );
		}
		void subscribe() {
			client->values.clear();
			client->subscribe( "path to/variable 2" );
			QCOMPARE( client->values[ "path to/variable 2" ], QString( "42" ) );
		}
		void getchanges() {
			protocol->updateVariable( "path to/variable 2", "21" );
			QCOMPARE( client->values[ "path to/variable 2" ], QString( "21" ) );
		}
		void unsubscribe() {
			client->unsubscribe( "path to/variable 2" );
			protocol->updateVariable( "path to/variable 2", "42" );
			QCOMPARE( client->values[ "path to/variable 2" ], QString( "21" ) );
			QCOMPARE( client->getvariable( "path to/variable 2" ), QString( "42" ) );
		}
};

#endif
