
#ifndef LANGUAGESCANNERTEST
#define LANGUAGESCANNERTEST

#include <QtTest/QtTest>

#include <netprotocol.h>
#include <QtCore>

#include "iobuffer.h"
#include "testnetprotocol.h"
#include "testnetprotocol.moc"

using namespace NSNetwork;

class NetProtocolTest : public QObject
{
    Q_OBJECT
    private:
        TestNetProtocol* protocol;
        IOBuffer buffer;
    protected slots:
        void setStatus(QString status) {
            protocol->updateStatus(status);
        }
    private slots:
        void initTestCase() {
            buffer.open( QIODevice::ReadOnly | QIODevice::WriteOnly );
            protocol = new TestNetProtocol( &buffer, this );
            connect(protocol, SIGNAL(setStatus(QString)), this, SLOT(setStatus(QString)));
            protocol->addToSpectrumList("Heights");
        }
        void cleanupTestCase() {
            delete protocol;
        }
        void cleanup() {
            QVERIFY( buffer.cleanBuffers() == 0 );
        }
        void simpletests_data() {
            QTest::addColumn<QByteArray>( "input" );
            QTest::addColumn<QByteArray>( "output" );

            QTest::newRow( "getvariableslist" )
                << QByteArray( "GET VARIABLES\x3" )
                << QByteArray( "UPDATE VARIABLES \"variable1\",\"path to/variable 2\"\x3" );
            QTest::newRow( "getvariable1type" )
                << QByteArray( "GET TYPE \"variable1\"\x3" )
                << QByteArray( "UPDATE TYPE \"variable1\" \"double\"\x3" );
            QTest::newRow( "getvariable1" )
                << QByteArray( "GET \"variable1\"\x3" )
                << QByteArray( "UPDATE \"variable1\" \"0.0815\"\x3" );
            QTest::newRow( "getboth" )
                << QByteArray( "GET \"variable1\"\x3GET \"path to/variable 2\"\x3" )
                << QByteArray( "UPDATE \"variable1\" \"0.0815\"\x3UPDATE \"path to/variable 2\" \"4711\"\x3" );
            QTest::newRow( "setvar2" )
                << QByteArray( "SET \"path to/variable 2\" \"5000\"\x3" )
                << QByteArray( "UPDATE \"path to/variable 2\" \"5000\"\x3" );
            QTest::newRow( "spectra" )
                << QByteArray( "GET SPECTRA\x3" )
                << QByteArray( "UPDATE SPECTRA \"Heights\"\x3" );
            QTest::newRow( "spectra/heights" )
                << QByteArray( "GET \"SPECTRUM/Heights\"\x3" )
                << QByteArray( "UPDATE \"SPECTRUM/Heights\" \"; Test-Heights\n-1 0\n-0.5 10\n0 20\n\"\x3" );
            QTest::newRow( "plainxml" )
                << QByteArray( "GET XML\x3" )
                << QByteArray( "UPDATE XML \"\"\x3" );
            QTest::newRow( "setxml" )
                << QByteArray( "SET XML \"<xml version=\"1.0\">\"\x3" )
                << QByteArray( "UPDATE XML \"<xml version=\"1.0\">\"\x3" );
            QTest::newRow( "status/unknown" )
                << QByteArray( "GET STATUS\x3" )
                << QByteArray( "UPDATE STATUS \"Unknown\"\x3" );
            QTest::newRow( "status/set start" )
                << QByteArray( "SET STATUS \"start\"\x3" )
                << QByteArray( "UPDATE STATUS \"start\"\x3" );
            QTest::newRow( "status/set start2" )
                << QByteArray( "SET STATUS \"start\"\x3" )
                << QByteArray( "UPDATE STATUS \"start\"\x3" );
            QTest::newRow( "status/set stop" )
                << QByteArray( "SET STATUS \"stop\"\x3" )
                << QByteArray( "UPDATE STATUS \"stop\"\x3" );
        }
        void simpletests() {
            QFETCH( QByteArray, input );
            QFETCH( QByteArray, output );
            buffer.addToIn( input );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), output.data() );
        }
        void listsubscriptions1() {
            buffer.addToIn( "GET SUBSCRIPTIONS\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE SUBSCRIPTIONS \"\"\x3" );
        }
        void subscription() {
            buffer.addToIn( "SUBSCRIBE \"variable1\"\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE \"variable1\" \"0.0815\"\x3" );
        }
        void update() {
            protocol->updateVariable( "variable1", "bla" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE \"variable1\" \"bla\"\x3" );
        }
        void listsubscriptions2() {
            buffer.addToIn( "GET SUBSCRIPTIONS\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE SUBSCRIPTIONS \"variable1\"\x3" );
        }
        void unsubscription() {
            buffer.addToIn( "UNSUBSCRIBE \"variable1\"\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "OK\x3" );
        }
        void listsubscriptions3() {
            buffer.addToIn( "GET SUBSCRIPTIONS\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE SUBSCRIPTIONS \"\"\x3" );
        }
};

#endif
