
#ifndef QTREEITEMNETCLIENTTEST_H
#define QTREEITEMNETCLIENTTEST_H

#include <QtTest/QtTest>
#include <QtCore>

#include <tiparameter.h>

#include <netclient.h>
#include <netprotocol.h>

#include "iobuffer.h"
#include "iobuffer.moc"

using namespace NSNetwork;

class QTreeItemNetClientTest : public QObject
{
	Q_OBJECT
	public:
		QTreeItemNetClientTest() : tree( 0 ), client( 0 ), protocol( 0 ), pipe( 0 ) {
		}
	private:
		NSCore::QTreeItem* tree;
		QTreeItemNetClient* client;
		QTreeItemNetProtocol* protocol;
		DebuggingPipe* pipe;
	private slots:
		void initTestCase() {
			pipe = new DebuggingPipe( this );
			QVERIFY( pipe->first()->write( "Bla" ) == 3 );
			QVERIFY( pipe->second()->write( "Blub" ) == 4 );
			QVERIFY( pipe->second()->readAll() == "Bla" );
			QVERIFY( pipe->first()->readAll() == "Blub" );
			tree = new NSCore::TiParameter( "", QVariant::Invalid, 0 );
			new NSCore::TiParameter( "bla", "blub", tree );
			new NSCore::TiParameter( "first", 10, tree );
			new NSCore::TiParameter( "second", 20.0, tree );
			new NSCore::TiParameter( "sub", 15, tree->getNamedChild( "first" ) );
			QCOMPARE( tree->childlist(), QStringList() << "bla" << "first" << "second" );
			//tree->debugPrint();
			protocol = new QTreeItemNetProtocol( tree, pipe->first() );
		}
		void cleanupTestCase() {
			if ( client )
				delete client;
			if ( protocol )
				delete protocol;
			if ( pipe )
				delete pipe;
			delete tree;
		}
		void cleanup() {
			QVERIFY( pipe->first()->bytesAvailable() == 0 );
			QVERIFY( pipe->second()->bytesAvailable() == 0 );
		}
		void initclient() {
			client = new QTreeItemNetClient( pipe->second(), this );
			QVERIFY( client->tree != 0 );
			//client->tree->debugPrint();
			QCoreApplication::processEvents();
			QCOMPARE( client->tree->childs(), tree->childs() );
			QCOMPARE( client->tree->childlist(), tree->childlist() );
		}
		void getvariables() {
			QStringList tmp = client->allvariables();
			QCOMPARE( tmp, QStringList() << "/" << "/bla" << "/first" << "/first/sub" << "/second" );
		}
		void getvar1() {
			QString tmp = client->getvariable( "/bla" );
			QCOMPARE( tmp, QString( "blub" ) );
		}
		void getvar2() {
			QCoreApplication::processEvents();
			QCOMPARE( client->tree->getNamedChild( "bla" )->guiValue(), QVariant( "blub" ) );
		}
		void setvar1() {
			QString t1 = client->getvariable( "/first/sub" );
			QCOMPARE( t1, QString( "15" ) );
			QString t2 = client->setvariable( "/first/sub", "42" );
			QCOMPARE( t2, QString( "42" ) );
		}
		void setvar2() {
			QVariant t1 = client->tree->getNamedChild( "first/sub" )->guiValue();
			QCOMPARE( t1.toInt(), 15 );
			client->setvariable( "/first/sub", "21" ); // This will (correctly so) not change the tree-item!
			int t2 = client->tree->getNamedChild( "first/sub" )->guiValue().toInt();
			QCOMPARE( t2, 15 );
		}
		void setvar3() {
			QCOMPARE( tree->getNamedChild( "first/sub" )->guiValue(), QVariant( 21 ) );
			client->tree->getNamedChild( "first/sub" )->guiSetValue( 42 );
			QCOMPARE( tree->getNamedChild( "first/sub" )->guiValue(), QVariant( 42 ) );
		}
		void checkupdates() {
			QCOMPARE( client->tree->getNamedChild( "first" )->guiValue(), QVariant( 10 ) );
			tree->getNamedChild( "first" )->guiSetValue( 11 );
			QTest::qWait( 550 );
			QCOMPARE( tree->getNamedChild( "first" )->guiValue(), QVariant( 11 ) );
			QCOMPARE( client->tree->getNamedChild( "first" )->guiValue(), QVariant( 11 ) );
		}
		void treechanged() {
			new NSCore::TiParameter( "update", 2.35482, tree->getNamedChild( "second" ) );
			QCoreApplication::processEvents();
			QCoreApplication::processEvents();
			QVERIFY( client->tree->getNamedChild( "second/update" ) );
		}
		void treechanged2() {
			new NSCore::TiParameter( "another update", 3.1416, tree->getNamedChild( "second/update" ) );
			QCoreApplication::processEvents( 0, 1000 );
			QVERIFY( client->tree->getNamedChild( "second/update/another update" ) );
		}
		void treechanged3() {
			delete tree->getNamedChild( "second/update/another update" );
			QCoreApplication::processEvents( 0, 1000 );
			QVERIFY( client->tree->getNamedChild( "second/update/another update" ) == 0 );
			QCOMPARE( client->tree->getNamedChild( "second/update" )->childs(), 0 );
		}
};

#endif
