
#ifndef LANGUAGESCANNERTEST
#define LANGUAGESCANNERTEST

#include <QtTest/QtTest>

#include <netprotocol.h>
#include <QtCore>

#include "iobuffer.h"
#include <tiparameter.h>

using namespace NSNetwork;

class QTreeItemNetProtocolTest : public QObject
{
    Q_OBJECT
    private:
        QTreeItemNetProtocol* protocol;
        IOBuffer buffer;
        NSCore::QTreeItem* tree;
    private slots:
        void initTestCase() {
            tree = new NSCore::TiParameter( "root", QVariant::Invalid, 0 );
            NSCore::TiParameter* bla = new NSCore::TiParameter( "test", QVariant::Invalid, tree );
            new NSCore::TiParameter( "var 1", 42, bla );
            new NSCore::TiParameter( "var 2", 42, bla );
            buffer.open( QIODevice::ReadOnly | QIODevice::WriteOnly );
            protocol = new QTreeItemNetProtocol( tree, &buffer );
        }
        void cleanupTestCase() {
            delete tree;
            delete protocol;
        }
        void cleanup() {
            QVERIFY( buffer.cleanBuffers() == 0 );
        }
        void simpletests_data() {
            QTest::addColumn<QByteArray>( "input" );
            QTest::addColumn<QByteArray>( "output" );

            QTest::newRow( "getvariableslist" )
                << QByteArray( "GET VARIABLES\x3" )
                << QByteArray( "UPDATE VARIABLES \"/root\",\"/root/test\",\"/root/test/var 1\",\"/root/test/var 2\"\x3" );
            QTest::newRow( "gettype1" )
                << QByteArray( "GET TYPE \"/test/var 1/\"\x3" )
                << QByteArray( "UPDATE TYPE \"/test/var 1/\" \"int\"\x3" );
            QTest::newRow( "gettypeinvalid" )
                << QByteArray( "GET TYPE \"/test/invalid/\"\x3" )
                << QByteArray( "UPDATE TYPE \"/test/invalid/\" \"invalid\"\x3" );
            QTest::newRow( "getvar1" )
                << QByteArray( "GET \"/test/var 1/\"\x3" )
                << QByteArray( "UPDATE \"/test/var 1/\" \"42\"\x3" );
            QTest::newRow( "getvar1.1" )
                << QByteArray( "GET \"test/var 1\"\x3" )
                << QByteArray( "UPDATE \"test/var 1\" \"42\"\x3" );
            QTest::newRow( "getboth" )
                << QByteArray( "GET \"/test/var 1\"\x3GET \"/test/var 2\"\x3" )
                << QByteArray( "UPDATE \"/test/var 1\" \"42\"\x3UPDATE \"/test/var 2\" \"42\"\x3" );
            QTest::newRow( "setvar2" )
                << QByteArray( "SET \"/test/var 2\" \"21\"\x3" )
                << QByteArray( "UPDATE \"/test/var 2\" \"21\"\x3" );
        }
        void simpletests() {
            QFETCH( QByteArray, input );
            QFETCH( QByteArray, output );
            buffer.addToIn( input );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), output.data() );
        }
        /*void subscription() {
            buffer.addToIn( "SUBSCRIBE \"/variable1\"\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE \"/variable1\" \"0.0815\"\x3" );
        }
        void update() {
            protocol->updateVariable( "/variable1", "bla" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "UPDATE \"/variable1\" \"bla\"\x3" );
        }
        void unsubscription() {
            buffer.addToIn( "UNSUBSCRIBE \"/variable1\"\x3" );
            QCoreApplication::processEvents();
            QCOMPARE( buffer.readOut().data(), "OK\x3" );
        }*/
};

#endif
