/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include <QtCore/QDebug>
#include <math.h>
#include <iostream>

#include "analyzers.h"

class MySpline : public Spline
{
	public:
		MySpline( double* y, int n, double yp1, double ypn ) : Spline( y, n, yp1, ypn ) {}

		double y( int n ) {
			if ( n<_n && n>=0 )
				return _y[ n ];
			return 0;
		}
};

int main(  int argc, char** argv )
{
	qDebug() << "Interpolation Test starting...";

	double* data = new double[ 20 ];
	data[ 0 ] = 1;
	data[ 1 ] = 0;
	data[ 2 ] = 1;
	data[ 3 ] = 1;
	data[ 4 ] = 0;
	data[ 5 ] = 0;
	data[ 6 ] = -1;
	data[ 7 ] = -4;
	data[ 8 ] = -6;
	data[ 9 ] = -5.5;
	data[ 10 ] = -4;
	data[ 11 ] = -3;
	data[ 12 ] = -2.5;
	data[ 13 ] = -2;
	data[ 14 ] = -1.5;
	data[ 15 ] = -1;
	data[ 16 ] = -0.5;
	data[ 17 ] = 0;
	data[ 18 ] = -1;
	data[ 19 ] = 0;

	for ( int i=0; i<19; ++i )
		std::cout << i << "\t:\t" << data[ i ] << std::endl;
	std::cout << std::endl << std::endl;

	PolynomInterpolate* interpolate = new PolynomInterpolate( 2 );
	interpolate->setData( data );
	interpolate->setMiddle( 2 );

	for ( double i=-1; i<=19.1; i+=0.05 ) {
		interpolate->setMiddle( int( i ) );
		std::cout << i << "\t:\t" << interpolate->polint( i ) << std::endl;
	}
	std::cout << std::endl << std::endl;
	for ( double i=-1; i<=19.1; i+=0.05 ) {
		interpolate->setMiddle( int( i+1 ) );
		std::cout << i << "\t:\t" << interpolate->polint( i ) << std::endl;
	}
	std::cout << std::endl << std::endl;

	MySpline* s = new MySpline( data, 20, 1e40, 1e40 );

	for ( double i=0; i<=19; i+=0.1 )
		std::cout << i << "\t:\t" << s->splint( i ) << std::endl;
	std::cout << std::endl << std::endl;

	//qDebug() << endl;

	/*Minimum* minimum = new Minimum( 3 );
	minimum->setData( data );
	minimum->setMinimum( 1 );

	qDebug() << minimum->golden_minimum().x << minimum->golden_minimum().y;
	qDebug() << minimum->golden_minimum( 7 ).x << minimum->golden_minimum( 7 ).y;*/

#if 0
	qDebug() << endl << endl;

	Interpolate* ip = new Interpolate();
	ip->setData( data );
	ip->setOrder( 3 );
	for ( int i=0; i<10; ++i ) {
		ip->setMiddle( i );
		for ( double j=-1; j<11.1; j+=0.1 )
			qDebug() << j << "\t" << ip->polint( j );
		qDebug() << endl;
	}
#endif

	qDebug() << "...finished.";
	return 0;
}

