
#ifndef QTREEITEMTEST
#define QTREEITEMTEST

#include <QtTest/QtTest>

//#include <analyzers.h>
#include <analyzers_gsl_cf.h>

using namespace NSCore;

class AbstractCfTest : public QObject
{
	Q_OBJECT
	protected:
		NSEvaluation::Analyzer* analyzer;
		NSCore::QTreeItem* tree;
		AbstractCfTest() {
			analyzer = 0;
			tree = new NSCore::TiParameter( "", QVariant::Invalid, 0 );
		}
		~AbstractCfTest() {
			delete analyzer;
			delete tree;
		}
		void cftest( QString resultname, double scale = -1, double fraction = 0.5, double targetaccuracy = 1E-3, bool expectfail = false ) {
			QString result = QString("channel 0/%1").arg(resultname);
			QString resultat = QString("channel 0/%1 at").arg(resultname);
			analyzer->configChanged();
			NSCore::AcqisitionDataSet data( 1, 50 );
			double xref = sqrt( -2 * pow(2, 2) * log(fraction));
			for ( double accuracy = 1; accuracy >= targetaccuracy; accuracy /= 10 ) {
				for ( double x = 24.5; x<=25.5; x+=0.1 ) {
					double* ch = data.getRawChannel( 0 );
					for ( int i=0; i<50; ++i ) {
						ch[i] =  scale * exp( - pow( i - x, 2 ) / ( 2 * pow( 2, 2 ) ) );
					}
					data.updateMinMax();
					data.addResult("channel 0/interpolated minimum at", x);
					data.addResult("channel 0/interpolated minimum", -1.0);
					data.addResult("channel 0/zeroline", 0.0);
					data.addResult("channel 0/integral", -10.0);
					analyzer->analyze( data );
					QVERIFY( ! isnan(data.getResult( resultat )) );
					QVERIFY( data.getResult( resultat ) != .0 );
					if (expectfail)
						QEXPECT_FAIL("", "CF-test is expected to fail.", Abort);
					if( fabs(x-xref - data.getResult(resultat)) > accuracy ) {
						qDebug() << "Accuracy =" << accuracy;
						qDebug() << x << data.getResult( resultat ) << xref << fabs(x - xref - data.getResult( resultat ));
						data.debugPrint();
					}
					QVERIFY( fabs((x-xref) - data.getResult(resultat)) < accuracy );
				}
			}
			//data.debugPrint();
		}
};

class ThresholdTest : public AbstractCfTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new ThresholdAnalyzer( tree );
			analyzer->configChanged();
		}
		void simple_data() {
			QTest::addColumn<double>("fraction");
			QTest::addColumn<double>("accuracy");
			double y = 0.1;
			for (; y<0.7; y+=0.1)
				QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E-2;
			//for (; y<0.9; y+=0.1)
			//	QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E0;
		}
		void simple() {
			QFETCH(double, fraction);
			QFETCH(double, accuracy);
			analyzer->config()->getParameter("threshold")->guiSetValue(-fraction);
			cftest("interpolated threshold", -1, fraction, accuracy);
		}
		// TODO extend the test
};
class CFTest : public AbstractCfTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new CFAnalyzer( tree );
			analyzer->config()->getParameter("interpolation order")->guiSetValue(4);
			analyzer->config()->getParameter("cf on minimum")->guiSetValue(true);
			analyzer->config()->getParameter("cf before extremum")->guiSetValue(true);
			analyzer->configChanged();
		}
		void simple_data() {
			QTest::addColumn<double>("fraction");
			QTest::addColumn<double>("accuracy");
			double y = 0.1;
			for (; y<0.7; y+=0.1)
				QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E-2;
			//for (; y<0.9; y+=0.1)
			//	QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E-1;
		}
		void simple() {
			QFETCH(double, fraction);
			QFETCH(double, accuracy);
			analyzer->config()->getParameter("fraction point")->guiSetValue(fraction);
			cftest("interpolated fraction point", -1, fraction, accuracy);
		}
		// TODO extend the test
};
class DynamicFractionTest : public AbstractCfTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new DynamicConstantFraction(tree);
			analyzer->config()->getParameter("interpolation order")->guiSetValue(4);
			analyzer->config()->getParameter("cf on minimum")->guiSetValue(true);
			analyzer->config()->getParameter("cf before extremum")->guiSetValue(true);
		}
		void simple_data() {
			QTest::addColumn<double>("weight");
			QTest::addColumn<double>("accuracy");
			double y = 0.01;
			for (; y<0.07; y+=0.01)
				QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E-2;
			//for (; y<0.9; y+=0.1)
			//	QTest::newRow(QString::number(y).toLocal8Bit()) << y << 1E-1;
		}
		void simple() {
			QFETCH(double, weight);
			QFETCH(double, accuracy);
			analyzer->config()->getParameter("weight")->guiSetValue(weight);
			cftest("dynamic fraction point", -1, weight*10, accuracy);
		}
};

#endif
// vim: noet
