
#ifndef QTREEITEMTEST
#define QTREEITEMTEST

#include <QtTest/QtTest>

#include <globallimiter.h>
#include <analyzers.h>
#include <analyzers_extrema.h>

using namespace NSCore;

class AbstractMinimumTest : public QObject
{
	Q_OBJECT
	protected:
		NSEvaluation::Analyzer* analyzer;
		NSCore::QTreeItem* tree;
		AbstractMinimumTest() {
			analyzer = 0;
			tree = new NSCore::TiParameter( "", QVariant::Invalid, 0 );
		}
		~AbstractMinimumTest() {
			delete analyzer;
			delete tree;
		}
		void extremumtest( QString resultname, double scale = -1, double targetaccuracy = 1E-3, bool expectfail = false ) {
			QString result = QString("channel 0/%1").arg(resultname);
			QString resultat = QString("channel 0/%1 at").arg(resultname);
			analyzer->configChanged();
			NSCore::AcqisitionDataSet data( 1, 50 );
			for ( double accuracy = 1; accuracy >= targetaccuracy; accuracy /= 10 ) {
				//qDebug() << accuracy;
				for ( double x = 23.5; x<=26.5; x+=0.1 ) {
					double* ch = data.getRawChannel( 0 );
					for ( int i=0; i<50; ++i ) {
						ch[i] =  scale * exp( - pow( i - x, 2 ) / ( 2 * pow( 2, 2 ) ) );
					}
					data.updateMinMax();
					//QCOMPARE( data.getResult( "channel 0/min" ), scale );
					//QCOMPARE( data.getResult( "channel 0/min at" ), floor(x) );
					analyzer->analyze( data );
					QVERIFY( data.getResult( result ) != .0 );
					if (expectfail)
						QEXPECT_FAIL("", "Extremumtest is expected to fail.", Abort);
					QVERIFY( fabs( data.getResult( result ) - scale ) < accuracy );
					if (expectfail)
						QEXPECT_FAIL("", "Extremumtest is expected to fail.", Abort);
					QVERIFY( fabs( data.getResult( resultat ) - x ) < accuracy );
				}
				//data.debugPrint();
			}
			//data.debugPrint();
		}
};
class LagrangeMinimumTest : public AbstractMinimumTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new InterpolatedMinimumAnalyzerLagrange( tree );
		}
		void minimum() {
			extremumtest("interpolated minimum", -1, 0.1);
		}
		void minimum2() {
			extremumtest("interpolated minimum", 2, 0.1, true);
		}
		void minimum3() {
			extremumtest("interpolated minimum", -2, 0.1);
		}
};

class InterpolatedMinimumTest : public AbstractMinimumTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new InterpolatedMinimumAnalyzer( tree );
		}
		void minimum() {
			extremumtest("interpolated minimum", -1, 1E-2);
		}
		void minimum2() {
			extremumtest("interpolated minimum", 2, 0.1, true);
		}
		void minimum3() {
			extremumtest("interpolated minimum", -2, 1E-2);
		}
};

class InterpolatedMaximumTest : public AbstractMinimumTest
{
	Q_OBJECT
	private slots:
		void initTestCase() {
			analyzer = new InterpolatedMaximumAnalyzer( tree );
		}
		void maximum() {
			extremumtest("interpolated maximum", 1, 1E-2);
		}
		void maximum2() {
			extremumtest("interpolated maximum", -2, 0.1, true);
		}
		void maximum3() {
			extremumtest("interpolated maximum", 2, 1E-2);
		}
};

#if 0
class QTreeItemTest : public QObject
{
	Q_OBJECT
	private:
		NamedTreeItem* tree;
	private slots:
		void initTestCase() {
			tree = new NamedTreeItem( "item", 0 );
			//tree->debugPrint();
		}
		void cleanupTestCase() {
			QPointer<NamedTreeItem> tmp = tree;
			//tree->debugPrint();
			delete tree;
			QVERIFY( tmp == 0 );
		}
		void tree_init() {
			QCOMPARE( tree->childs(), 0 );
			QCOMPARE( tree->objectName(), QString("item") );
			tree->init();
			//tree->debugPrint();
			QCOMPARE( tree->childs(), 2 );
		}
		void first_appendchilds() {
			NamedTreeItem* tmp = tree->child<NamedTreeItem>( "first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void first_second_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 2 );
		}
		void second_first_get() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "second/first" );
			QVERIFY( ! tmp );
		}
		void first_second_first_appendchilds() {
			NamedTreeItem* tmp = tree->getNamedChild<NamedTreeItem>( "first/second/first" );
			QVERIFY( tmp );
			tmp->init();
			QCOMPARE( tmp->childs(), 3 );
		}
		void recursive_delete() {
			QPointer<NamedTreeItem> tmp = tree->getNamedChild<NamedTreeItem>( "first/second" );
			QVERIFY( tmp );
			delete tree->getNamedChild( "first/second" );
			QVERIFY( tmp == 0 );
			//tree->debugPrint();
		}
		void pointer() {
			QTreeItemPointer p( tree->getNamedChild( "first" ) );
			QVERIFY( p == tree->getNamedChild( "first" ) );
			QPointer<NamedTreeItem> p2( tree->getNamedChild<NamedTreeItem>( "first" ) );
			delete tree->getNamedChild( "first" );
			QVERIFY( p2 == 0 );
			QVERIFY( p == 0 );
		}
		/*void xml_finish() {
			QDomDocument doc( "bla" );
			doc.appendChild( tree->toXml( doc ) );
			qDebug() << doc.toString();
		}*/
};
#endif

#endif
