/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "scriptintrospection.h"
#include "scriptintrospection.moc"

#include "scriptintrospection_base.h"

#include <QtCore/QDebug>

#include <QtScript/QtScript>

#include "scriptenvironment.h"

using namespace NSScript;

ScriptIntrospection::ScriptIntrospection( ScriptEnvironment* e, QWidget* p ) : QWidget( p ), _env( e ) {
	_base = new Ui::ScriptIntrospection_Base();
	_base->setupUi( this );
}

void ScriptIntrospection::on_btnUpdate_clicked() {
	qDebug() << "ScriptIntrospection::on_btnUpdate_clicked()";
	_base->tree->clear();
	//tree->setHeaderLabels( QStringList() << "Name" << "Value" << "Flags" );

	QTreeWidgetItem* top = _base->tree->invisibleRootItem();
	foreach( QObject* obj, _env->_objects )
		addQObject( obj, top );

	_base->tree->resizeColumnToContents( 0 );
}

void ScriptIntrospection::addQObject( QObject* obj, QTreeWidgetItem* top ) {
	QTreeWidgetItem* item = new QTreeWidgetItem( top, QStringList() << obj->objectName() );
	item->setToolTip( 0, QString( "<qt><b>Name:</b> %1<br /><b>Type:</b> %2</qt>" ).arg( obj->objectName() ).arg( obj->metaObject()->className() ) );
	addProperties( obj, item );
	addMethods( obj, item );
	foreach( QObject* child, obj->children() )
		addQObject( child, item );
}
void ScriptIntrospection::addProperties( QObject* obj, QTreeWidgetItem* top ) {
	top = new QTreeWidgetItem( top, QStringList() << "Properties" );
	const QMetaObject* meta = obj->metaObject();
	for ( int i=0; i<meta->propertyCount(); ++i ) {
		QMetaProperty prop = meta->property( i );
		QString flags = "";
		if ( prop.isReadable() ) flags += "Read ";
		if ( prop.isWritable() ) flags += "Write ";
		//if ( prop.isValid() ) flags += "Valid ";
		if ( prop.isResettable() ) flags += "Reset ";
		if ( prop.isScriptable() ) flags += "Scriptable ";
		if ( prop.isEnumType() ) flags += "EnumType ";
		if ( prop.isFlagType() ) flags += "FlagType ";
		QString value = obj->property( prop.name() ).toString();
		QTreeWidgetItem* tmp = new QTreeWidgetItem( top, QStringList() << QString( "%1: %2" ).arg( prop.name() ).arg( value ) );
		tmp->setToolTip( 0, QString( "<qt><b>Property Name:</b>%1<br /><b>Value:</b> %2<br /><b>Flags:</b> %3</qt>" ).arg( prop.name() ).arg( value.replace( "<","&lt;" ) ).arg( flags ) );
	}
}
void ScriptIntrospection::addMethods( QObject* obj, QTreeWidgetItem* top ) {
	top = new QTreeWidgetItem( top, QStringList() << "Methods" );
	const QMetaObject* meta = obj->metaObject();
	for ( int i=0; i<meta->methodCount(); ++i ) {
		QMetaMethod method = meta->method( i );
		QString type = "Method";
		if ( method.methodType() == 1 )
			type = "Signal";
		if ( method.methodType() == 2 )
			type = "Slot";
		QString flags = method.typeName();
		/// Arguments (types,names) These are already visible in the first column.
		/*QList<QByteArray> argnames = method.parameterNames();
		QList<QByteArray> argtypes = method.parameterTypes();
		for ( int i=0; i<qMin( argnames.size(), argtypes.size() ); ++i ) {
			//qDebug() << " Argument" << i << ": name" << argnames.at( i ) << "\ttype" << argtypes.at( i );
			//args.append( QPair<QString,QString>( argtypes.at( i ), argnames.at( i ) ) );
			flags += argtypes.at( i ) + ": " + argnames.at( i ) + ", ";
		}*/
		QTreeWidgetItem* tmp = new QTreeWidgetItem( top, QStringList() << method.signature() );
		tmp->setToolTip( 0, QString( "<qt><b>Method:</b> %1 (%2)<br /><b>Return type:</b> %3</qt>" ).arg( method.signature() ).arg( type ).arg( flags ) );
	}
}

