/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SCRIPTWORKSPACE_H
#define SCRIPTWORKSPACE_H

#include <QtGui/QMainWindow>

class QMdiArea;

namespace NSScript {

	class ScriptEnvironment;
	class ScriptSnippet;
	class ScriptIntrospection;

	/**
	 * @todo dox!
	 * @todo Private pointer to store all the private parts and keep the public interface clean.
	 */
	class ScriptWorkSpace : public QMainWindow
	{
		Q_OBJECT
		public:
			ScriptWorkSpace( ScriptEnvironment*, QWidget* =0 );

		signals:
			void visibilityChanged( bool );

		public slots:
			void setVisible( bool );

		private slots:

			/**
			 * @brief Create a new and empty script snippet
			 */
			void newCodeSnippet();

			/**
			 * @brief Load a .js-file as new script snippet
			 */
			void loadCodeSnippet();

			void closeActiveSnippet();

			void closeAllSnippets();

			void newCodeSnippet( ScriptSnippet* );

			/**
			 * @todo Fix the deletion of subwindows...
			 */
			void removedSnippet( ScriptSnippet* );

			void executeActiveSnippet();

			void stopScript();

			void restoreSnippets();

		private:
			ScriptEnvironment* _env;
			QMdiArea* _mdiarea;
			ScriptIntrospection* _introspec;
	};

};

#endif // SCRIPTWORKSPACE_H

