
#ifndef TWODIMSPECTRUMMODELTEST
#define TWODIMSPECTRUMMODELTEST

#include <QtTest/QtTest>
#include <cmath>

#include <spectrummodel.h>

using namespace NSSpectrum;

class TwoDimSpectrumModelTest : public QObject
{
	Q_OBJECT
	private:
		TwoDimSpectrumModel* model;
		double min, max;
	private slots:
		void initTestCase() {
			model = new TwoDimSpectrumModel( this );
			QVERIFY( model );
			QVERIFY( model->config() );
			QCOMPARE( model->dimensions(), 2 );
			QVERIFY( ! model->supportedFilters().isEmpty() );
		}
		void cleanupTestCase() {
		}
		void setupChannels() {
			model->config()["X-Axis/Minimum"]->setValue( min = 0 );
			model->config()["X-Axis/Maximum"]->setValue( max = 1 );
			model->config()["Y-Axis/Minimum"]->setValue( min );
			model->config()["Y-Axis/Maximum"]->setValue( max );
			model->config()["X-Axis/Channels"]->setValue( 3 );
			model->config()["Y-Axis/Channels"]->setValue( 3 );
			QCOMPARE( model->rowCount(), 3 );
			for ( double i=0; i<model->columnCount(); ++i ) {
				QCOMPARE( model->headerData( int( i ), Qt::Vertical ).toDouble(), i/model->columnCount() );
			}
			for ( double i=0; i<model->rowCount(); ++i ) {
				QCOMPARE( model->headerData( int( i ), Qt::Vertical ).toDouble(), i/model->rowCount() );
			}
		}
		void checkForEmptyModel() {
			for ( int i=0; i<model->rowCount(); ++i ) {
				QModelIndex idx = model->index( i, 0 );
				QCOMPARE( model->data( idx ).toDouble(), 0.0 );
			}
		}
		void addDataAndCheck_data() {
			QTest::addColumn<double>( "d" );
			QList<double> tmp;
			tmp << 0.1 << 0.5 << 0.7 << 0.0 << 1.0 << -1.0 << 5.0;
			foreach ( double n, tmp ) {
				QTest::newRow( qPrintable( QString::number( n ) ) ) << n;
			}
		}
		void addDataAndCheck() {
			QFETCH( double, d );
			model->data( QList<double>() << d << d );
			if ( d >= min && d < max )
				QCOMPARE( model->data( model->index( int(d*3), int(d*3) ) ).toInt(), 1 );
		}
		void benchmarkAddData() {
			srand( QDateTime::currentDateTime().toTime_t() );
			QBENCHMARK {
				model->data( QList<double>() << rand() * 1.0 / RAND_MAX * max );
			}
		}
		void clearSpectrum() {
			model->clearSpectrum();
			for ( int i=0; i<model->rowCount(); ++i ) {
				QCOMPARE( model->data( model->index( i, 0 ) ).toInt(), 0 );
			}
		}
		void benchmarkAddDataBig_data() {
			QTest::addColumn<double>( "channels" );
			for ( int i=1; i<9; ++i ) {
				QTest::newRow( qPrintable( QString::number( pow( 2, i ) ) ) ) << pow( 2, i );
			}
		}
		void benchmarkAddDataBig() {
			QFETCH( double, channels );
			model->config()[ "X-Axis/Channels" ]->setValue( channels );
			model->config()[ "Y-Axis/Channels" ]->setValue( channels );
			QBENCHMARK {
				model->data( QList<double>() << rand()*1.0/RAND_MAX << rand()*1.0/RAND_MAX );
			}
		}
		/*
		void benchmark_getresult_data() {
			QTest::addColumn<QString>("result");
			foreach ( QString name, data->getResultNames() ) {
				QTest::newRow(qPrintable(name)) << qPrintable(name);
			}
		}
		void benchmark_getresult() {
			QFETCH( QString, result );
			QBENCHMARK{ data->getResult( result ); }
		}
		*/
};

#endif
