/*
    Copyright 2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "qvxi11.h"
#include "qvxi11.moc"

#include <QtCore/QDebug>
#include <QtCore/QThread>
#include <QtNetwork/QtNetwork>
#include <math.h> // for pow()

//#include "vxi11.h"
#include "vxi11_xdr.c"
#include "vxi11_clnt.c"
//#include "vxi11irqproto.h"
#include "vxi11irqproto_xdr.c"
#include "vxi11irqproto_svc.c"

void* device_intr_srq_1_svc( Device_SrqParms *parms, struct svc_req *req ) {
	qDebug() << "device_intr_srq_1_svc(" << parms << "," << req << ")";
	return 0;
}

class InterruptThread : public QThread
{
	public:
		InterruptThread( QObject* p ) : QThread( p ), myport( 0 ), stop( false ) {
		}

		int myport;
		bool stop;

		void run() {
			register SVCXPRT *transp;

			pmap_unset ( DEVICE_INTR, DEVICE_INTR_VERSION );

			transp = svctcp_create( RPC_ANYSOCK, 0, 0 );
			if ( transp == NULL ) {
				fprintf ( stderr, "%s", "cannot create tcp service." );
				return; //exit( 1 );
			}
			if ( !svc_register( transp, DEVICE_INTR, DEVICE_INTR_VERSION, device_intr_1, IPPROTO_TCP ) ) {
				fprintf ( stderr, "%s", "unable to register (DEVICE_INTR, DEVICE_INTR_VERSION, tcp)." );
				return; //exit( 1 );
			}

			qDebug() << " My interrupt port is" << transp->xp_port;
			myport = transp->xp_port;
			//qDebug() << " My remote address is" << QHostAddress( transp->xp_raddr.sin_addr.s_addr );

			//svc_run ();
			struct timeval tv;
			tv.tv_sec = 0;
			tv.tv_usec = 100;
			int retval = 0;
			fd_set rfds;
			FD_ZERO( &rfds );
			FD_SET( transp->xp_sock, &rfds );

			while ( !stop ) {
				retval = select( 1, &rfds, NULL, NULL, &tv );
				if ( retval > 0 ) {
					qDebug() << "InterruptThread::run() retval =" << retval;
					svc_getreqset( &rfds );
				}
				//svc_getreq( transp->xp_sock );
				//usleep( 100 );
			}
			fprintf ( stderr, "%s\n", "svc_run returned" );
		}
};

class QVxi11::QVxi11_Private {
	public:
		CLIENT* rpcClient;

		Device_Link lid;
		int abortPort;
		int maxRecvSize;

		InterruptThread* irqthread;

		QVxi11_Private()
			: rpcClient( 0 )
		{
		}
};

QVxi11::QVxi11( QObject* p ) : QObject( p ), d( new QVxi11::QVxi11_Private() ) {
	qDebug() << "QVxi11::QVxi11(" << p << ")";
//	d->irqthread = new InterruptThread( this );
//	d->irqthread->start();
}
QVxi11::~QVxi11() {
	qDebug() << "QVxi11::~QVxi11()";
//	d->irqthread->stop = true;
//	d->irqthread->wait();
//	delete d->irqthread;
	if ( d->rpcClient )
		clnt_destroy( d->rpcClient );
	delete d;
	qDebug() << "QVxi11::~QVxi11() finished";
}

void QVxi11::connectToHost( const QHostAddress& address ) {
	qDebug() << "QVxi11::connectToHost(" << address << ")";
	if ( ! d->rpcClient ) {
		d->rpcClient = clnt_create( address.toString().toStdString().c_str(), DEVICE_CORE, DEVICE_CORE_VERSION, "tcp" );
		if ( d->rpcClient ) {
			Create_LinkParms parms;
			parms.clientId = 0;
			parms.lockDevice = false;
			parms.lock_timeout = 0;
			parms.device = "inst0";
			Create_LinkResp* resp = create_link_1( &parms, d->rpcClient );
			qDebug() << " Link-Create error =" << resp->error;
			qDebug() << "  linkId =" << resp->lid << " abortPort =" << resp->abortPort << " maxRecvSize =" << resp->maxRecvSize;
			d->lid = resp->lid;
			d->abortPort = resp->abortPort;
			d->maxRecvSize = resp->maxRecvSize;
			emit connected();

#if 0
			Device_EnableSrqParms parms3;
			parms3.lid = d->lid;
			parms3.enable = true;
			Device_Error* error = device_enable_srq_1( &parms3, d->rpcClient );
			qDebug() << " Enable SRQ error:" << ( error?error->error:1000 );
			Device_RemoteFunc parms2;
			//sockaddr_in myaddr;
			//get_myaddress( &myaddr );
			//parms2.hostAddr = myaddr.sin_addr.s_addr;
			parms2.hostAddr = QHostAddress( "10.0.0.10" ).toIPv4Address();
			parms2.hostPort = d->irqthread->myport;
			parms2.progNum = DEVICE_INTR;
			parms2.progVers = DEVICE_INTR_VERSION;
			parms2.progFamily = DEVICE_TCP;
			//qDebug() << QHostAddress( parms2.hostAddr /*myaddr.sin_addr.s_addr*/ );
			/*Device_Error* */error = create_intr_chan_1( &parms2, d->rpcClient );
			qDebug() << " IRQ-channel error:" << error->error;
			error = device_enable_srq_1( &parms3, d->rpcClient );
			qDebug() << " Enable SRQ error:" << ( error?error->error:1000 );
#endif
			qDebug() << " Connected...";
			return;
		} else
			qDebug() << " rpcClient failed to create...";
	} else
		qDebug() << " Already a client existing and connected...";
}

QString QVxi11::sendCmd( QString cmd ) {
	//qDebug() << "QVxi11::sendCmd(" << cmd << ")";
	if ( ! cmd.length() || d->rpcClient == 0 )
		return "";

	if ( ! cmd.endsWith( "\n" ) )
		cmd.append( "\n" );

	Device_WriteParms wparms;
	wparms.lid = d->lid;
	wparms.io_timeout = 0;
	wparms.lock_timeout = 0;
	wparms.flags = 0;
	wparms.data.data_val = cmd.toAscii().data();
	wparms.data.data_len = cmd.length();
	Device_WriteResp* wresp = device_write_1( &wparms, d->rpcClient );
	if ( wresp->error != 0 ) {
		qDebug() << " written. error =" << wresp->error << " size =" << wresp->size;
		return "";
	}

	Device_ReadParms rparms;
	rparms.lid = d->lid;
	rparms.io_timeout = 0;
	rparms.lock_timeout = 0;
	rparms.flags = 0;
	rparms.requestSize = int( pow( 2, 16 ) );
	rparms.termChar = '\n';
	Device_ReadResp* rresp = device_read_1( &rparms, d->rpcClient );
	//printf( "%s\n", rresp->data.data_val );
	if ( rresp->error != 0 )
		qDebug() << " read. error =" << rresp->error << " reason =" << rresp->reason << " data_len:" << rresp->data.data_len << rresp->data.data_val;

	/*for ( int i=0; i<rresp->data.data_len; ++i ) {
		qDebug() << i << ":" << QString::number( rresp->data.data_val[ i ] );
	}*/
	QString ret = QString::fromAscii( rresp->data.data_val, rresp->data.data_len );
	while ( ret.endsWith( '\n' ) || ret.endsWith( QChar( 0 ) ) )
		ret.chop( 1 );
	//delete wresp;
	//delete rresp;
	return ret.simplified();
}

