/*
    Copyright 2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/
#ifndef QVXI_TEST_H
#define QVXI_TEST_H

#include <QtGui/QtGui>

#include "qvxi11.h"

class QVxiTest : public QWidget
{
	Q_OBJECT
	public:
		QVxiTest( QWidget* p )
			: QWidget( p )
			, vxi( 0 )
		{
			qDebug() << "QVxiTest::QVxiTest(" << p << ")";
			QGridLayout* layout = new QGridLayout( this );

			list = new QListWidget( this );
			layout->addWidget( list, 0, 0, 4, 1 );

			QPushButton* btnStart = new QPushButton( "Start VXI-11", this );
			connect( btnStart, SIGNAL( clicked() ), this, SLOT( startVXI() ) );
			layout->addWidget( btnStart, 0, 1 );
			QPushButton* btnExec = new QPushButton( "Execute commands", this );
			connect( btnExec, SIGNAL( clicked() ), this, SLOT( sendCommands() ) );
			layout->addWidget( btnExec, 1, 1 );
			QPushButton* btnQuit = new QPushButton( "Quit", this );
			connect( btnQuit, SIGNAL( clicked() ), this, SLOT( close() ) );
			layout->addWidget( btnQuit, 2, 1 );

			vxi = new QVxi11( this );

			QStringList commands;
			commands << "*IDN?";
			//commands << "SENSE:INPUT1:STATE?" << "SENSE:INPUT2:STATE?" << "SENSE:INPUT3:STATE?" << "SENSE:INPUT4:STATE?";
			//commands << "sense:voltage1:protection?" << "sense:voltage3:protection?";

			commands << "status:operation:event?";

			//commands << "sense:sweep:segment:mode?" << "sense:sweep:segment:count?" << "sense:sweep:segment:view?";
			//commands << "sense:sweep:segment:view 1" << "sense:sweep:segment:view?";
			commands << "sense:sweep:points?";

			//commands << "trace:automatic:synchronize?" << "trace:automatic:zoom?";
			commands << /*"format:data?" <<*/ "format:data int,8;data?";

			//commands << "arm?" << "arm:source?" << "arm 1;:arm?" << "system:error:all?";
			commands /*<< "initiate?"*/ << "initiate:continuous?" << "init;:system:error:all?";
			commands << "*opc?" << "*TRG" << "*WAI";// << "*OPC?";

			commands << "trace:preamble? inp1";// << "system:error:all?";
			commands << "trace:preamble?";// << "system:error:all?";

			commands << "TRACE:DATA? INP1";// << "system:error:all?";
			commands << "TRACE:DATA?INP1";// << "system:error:all?";
			commands << "TRACE? INP1";// << "system:error:all?";
			commands << "TRACE? REF1";// << "system:error:all?";

			commands << "trace:preamble? inp1;:trace? input1";

			//commands << "trace:load:input1? 0,100, 100";
			list->addItems( commands );

			qDebug() << "QVxiTest::QVxiTest finished.";
		}
		~QVxiTest() {
			qDebug() << "QVxiTest::~QVxiTest()";
		}

	public slots:
		void startVXI() {
			qDebug() << "QVxiTest::startVXI()";
			vxi->connectToHost( QHostAddress( "192.168.25.120" ) );
		}

		void sendCommands() {
			qDebug() << "QVxiTest::sendCommands()";
			/*QStringList commands;
			commands << "*IDN?";
			foreach( QString cmd, commands ) {
				QString answer = vxi->sendCmd( cmd );
				qDebug() << "| " << cmd << ": _" << answer << "_" << answer.size();
			}*/
			QString cmd = list->currentItem()->text();
			QString answer = vxi->sendCmd( cmd );
			qDebug() << "| " << cmd << ": _" << answer << "_" << answer.size();
			qDebug() << " Last error(s):" << vxi->sendCmd( "system:error:all?" );
		}

	private:
		QVxi11* vxi;
		QListWidget* list;
};

#if 0
#endif

#endif
