/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include <QtCore/QCoreApplication>

//#include <netclient.h>
//#include <acqiris.h>

#include <acqiris_thread.h>
#include <net_thread.h>
#include <worker_thread.h>

int main( int argc, char* argv[] )
{
	QCoreApplication* app = new QCoreApplication( argc, argv );

	//NetClient * client = new NetClient( app );

	AcqirisThread* acqiristhread = new AcqirisThread( app );
	NetThread* netthread = new NetThread( app );
	WorkerThread* workthread = new WorkerThread( app );

	QObject::connect( netthread->path( "/quit" ), SIGNAL( gotMessage() ), acqiristhread, SLOT( quit() ) );
	QObject::connect( netthread->path( "/quit" ), SIGNAL( gotMessage() ), netthread, SLOT( quit() ) );
	QObject::connect( netthread->path( "/quit" ), SIGNAL( gotMessage() ), workthread, SLOT( quit() ) );
	QObject::connect( netthread->path( "/quit" ), SIGNAL( gotMessage() ), app, SLOT( quit() ) );

	QObject::connect( acqiristhread, SIGNAL( gotData( NSAcqiris::ChannelData ) ),
	                  workthread, SLOT( getData( NSAcqiris::ChannelData ) ) );
	QObject::connect( workthread, SIGNAL( sendMessage( QString, QVariant ) ),
	                  netthread, SLOT( sendData( QString, QVariant ) ) );
	QObject::connect( netthread->path( "/filter/status" ), SIGNAL( gotMessage() ),
	                  workthread, SLOT( sendStatus() ) );

	QObject::connect( netthread->path( "/digitizer/start" ), SIGNAL( gotMessage() ),
	                  acqiristhread->acqiris()->digitizer( 0 ), SLOT( startAcqisition() ) );
	QObject::connect( netthread->path( "/digitizer/stop" ), SIGNAL( gotMessage() ),
	                  acqiristhread->acqiris()->digitizer( 0 ), SLOT( stopAcqisition() ) );
	QObject::connect( netthread->path( "/digitizer/calibrate" ), SIGNAL( gotMessage() ),
	                  acqiristhread->acqiris()->digitizer( 0 ), SLOT( calibrate() ) );
	QObject::connect( netthread->path( "/digitizer/singleshot" ), SIGNAL( gotMessage( bool ) ),
	                  acqiristhread->acqiris()->digitizer( 0 ), SLOT( singleShot( bool ) ) );

	QObject::connect( netthread->client(), SIGNAL( disconnected() ), acqiristhread, SLOT( quit() ) );
	QObject::connect( netthread->client(), SIGNAL( disconnected() ), netthread, SLOT( quit() ) );
	QObject::connect( netthread->client(), SIGNAL( disconnected() ), workthread, SLOT( quit() ) );
	QObject::connect( netthread->client(), SIGNAL( disconnected() ), app, SLOT( quit() ) );

	acqiristhread->start();
	netthread->start();
	workthread->start();
	int ret = app->exec();

	delete app;

	return ret;
}

