/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include "clientmodel.h"
#include "clientmodel.moc"
#include "clientmodel_private.h"

#include <QtGui/QColor>
#include <QtCore/QDebug>

using namespace NSClientModel;

ClientModel::ClientModel( QObject* p ) : QAbstractItemModel( p ), _rootItem( new TreeItem( "ROOT" ) ) {
	qDebug( "ClientModel::ClientModel( QObject* %p )", p );
	_testItem = new TreeItem( "Computer", _rootItem );
	QMap<QString,QVariant> map;
	new FilterItem( map, _testItem );
}
ClientModel::~ClientModel() {
	qDebug( "ClientModel::~ClientModel()" );
	delete _rootItem;
	_rootItem = 0;
}

QModelIndex ClientModel::index( int row, int col, const QModelIndex& parent ) const {
	//qDebug( "ClientModel::index( int %i, int %i, const QModelIndex& index )", row, col );
	//qDebug() << parent;
	TreeItem* parentItem;
	if ( !parent.isValid() )
		parentItem = _rootItem;
	else
		parentItem = static_cast<TreeItem*>( parent.internalPointer() );

	TreeItem* childItem = static_cast<TreeItem*>( parentItem->child( row ) );
	if ( childItem )
		return createIndex( row, col, childItem );
	else
		return QModelIndex();
}

QModelIndex ClientModel::parent( const QModelIndex& index ) const {
	//qDebug() << "ClientModel::parent( const QModelIndex& " << index << ")";
	if ( !index.isValid() )
		return QModelIndex();

	TreeItem* childItem = static_cast<TreeItem*>( index.internalPointer() );
	if ( childItem ) {
		TreeItem* parentItem = static_cast<TreeItem*>( childItem->parent() );

		if ( parentItem == _rootItem )
			return QModelIndex();

		if ( parentItem )
			return createIndex( parentItem->row(), 0, parentItem );
	}
	return QModelIndex();
}

int ClientModel::rowCount( const QModelIndex& parent ) const {
	//qDebug() << "ClientModel::rowCount( const QModelIndex& " << parent << ")";
	TreeItem* parentItem;
	if ( !parent.isValid() )
		parentItem = _rootItem;
	else
		parentItem = static_cast<TreeItem*>( parent.internalPointer() );

	//qDebug( " will return %i", parentItem->childcount() );
	return parentItem->childs();
}

int ClientModel::columnCount( const QModelIndex& ) const {
	//qDebug( "ClientModel::columnCount( const QModelIndex& )" );
	return 1;
}

QVariant ClientModel::data( const QModelIndex& index, int role ) const {
	//qDebug( "ClientModel::data( const QModelIndex&, int %i )", role );
	//qDebug() << index;
	if ( !index.isValid() )
		return QVariant();

/*	if ( role == Qt::BackgroundColorRole ) {
		return QColor( 200,255,200 );
	}*/

	if ( role == Qt::DisplayRole ) {
		TreeItem* item = static_cast<TreeItem*>( index.internalPointer() );
		return item->data();
	}

	return QVariant();
}

bool ClientModel::hasChildren( const QModelIndex& index ) const {
	//qDebug() << "ClientModel::hasChildren( const QModelIndex& " << index << " )";
	TreeItem* tmp = static_cast<TreeItem*>( index.internalPointer() );
	if ( tmp )
		return tmp->childs();
	if ( index.row() == -1 )
		return true;
	return false;
}

void ClientModel::newClient( EPOServerClient* client ) {
	qDebug( "ClientModel::newClient( EPOServerClient* %p )", client );
	QModelIndex rootindex = createIndex( 0, 0, _testItem );
	beginInsertRows( rootindex, _testItem->childs(), _testItem->childs() );
	new ComputerItem( client, _testItem );
	qDebug( "_rootItem has now %i childs", _testItem->childs() );
	endInsertRows();
	qDebug( "!- endInsertRows() finished" );
}
void ClientModel::removeClient( EPOServerClient* client ) {
	qDebug( "ClientModel::removeClient( EPOServerClient* %p )", client );
	ComputerItem* comp =0;
	int i=0;
	while ( i < _testItem->childs() && comp == 0 ) {
		comp = dynamic_cast<ComputerItem*>( _testItem->child( i ) );
		++i;
	}
	if ( comp ) {
		qDebug( "Found a ComputerItem-Child %p",comp );
		QModelIndex rootindex = createIndex( 0, 0, _testItem );
		beginRemoveRows( rootindex, i, i );
		_testItem->removeChild( comp );
		endRemoveRows();
	}
}

