/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */

#ifndef CLIENTMODEL_PRIVATE_H
#define CLIENTMODEL_PRIVATE_H

#include <QtCore/QList>
#include <QtCore/QDebug>
#include <QtCore/QVariant>
#include <QtCore/QPointer>
#include <QtCore/QObject>

//class EPOServerClient;
#include <net_server.h>

#include <abstracttreeitem.h>

namespace NSClientModel {

	class TreeItem : public NSCore::AbstractTreeItem {
		public:
			TreeItem( QVariant data, TreeItem* parent=0 ) : AbstractTreeItem( parent ), _data( data ) { 
				//qDebug( "TreeItem::TreeItem( TreeItem* parent %p )", parent );
				if ( parent )
					parent->addChild( this );
			}

			virtual ~TreeItem() {
				qDebug() << "TreeItem::~TreeItem()" << _data;
			}
	
/*			int row() const {
				//qDebug( "TreeItem::row()" );
				if ( _parent ) {
					TreeItem* tmp = const_cast<TreeItem*>( this );
					if ( tmp )
						return _parent->_childs.indexOf( tmp );
				}
				return 0;
			}*/

			void newChild() {
				new TreeItem( QVariant(), this );
			}
			void removeChild( TreeItem* n ) {
				delete n;
				_childs.removeAll( n );
			}
			void removeAllChilds() {
				qDeleteAll( _childs );
			}

			virtual QVariant data() const {
				return _data;
			}
		private:
			void addChild( TreeItem* n ) {
				//qDebug( "TreeItem::addChild( TreeItem* %p )", n );
				_childs.push_back( n );
				//qDebug( " has now %i childs", _childs.size() );
			}

		protected:
			QString debug() const {
				return "TreeItem";
			}
			QVariant _data;
	};

	class ComputerItem : public QObject, public TreeItem
	{
		Q_OBJECT
		public:
			ComputerItem( EPOServerClient*, TreeItem* );
			~ComputerItem() { }

			QVariant data() const {
				if ( _client )
					return QString( "Lab-PC '%1'" ).arg( _client->host() );
				return TreeItem::data();
			}
		public slots:
			void statusModules( QVariant );
		private:
			QPointer<EPOServerClient> _client;
			TreeItem* _filters;
			TreeItem* _modules;
	};

	class FilterItem : public QObject, public TreeItem
	{
		Q_OBJECT
		public:
			FilterItem( QMap<QString,QVariant>, TreeItem* );

			QList<QString> params() const { return _params.keys(); }
			QVariant value( QString ) const;
		public:
			void setValue( QString, QVariant );
			void setValue( QString, int );
			void setValue( QString, double );
		private:
			QMap<QString,QVariant> _params;
	};

};

#endif // CLIENTMODEL_PRIVATE_H

