/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */

#ifndef MAIN_H
#define MAIN_H

#include <QtCore/QObject>

#include <acqiris_datatypes.h>
#include <serverui.h>
#include <net_server.h>
#include "myosciwidget.h"

#include <clientmodel.h>
#include <clientdelegate.h>

class Clientview : public QWidget, private Ui::ServerUi {
	Q_OBJECT
	public:
		Clientview( QWidget* p=0 ) : QWidget( p ), _server( new EPOServer( this ) ), _model( new ClientModel( this ) ) {
			Ui::ServerUi::setupUi( this );
			connect( clientList, SIGNAL( itemSelectionChanged() ), this, SLOT( itemSelectionChanged() ) );
			qDebug() << "connect returned " << ( connect( btnQuitClient, SIGNAL( clicked() ), this, SLOT( btnQuitClientClicked() ) )?"true":"false" );
			connect( btnViewOsci, SIGNAL( clicked() ), this, SLOT( btnViewOsciClicked() ) );
			connect( btnViewFilter, SIGNAL( clicked() ), this, SLOT( btnViewFilterClicked() ) );

			connect( _server, SIGNAL( newConnection( QString ) ), this, SLOT( newClient( QString ) ) );
			connect( _server, SIGNAL( newConnection( EPOServerClient* ) ),
			         _model, SLOT( newClient( EPOServerClient* ) ) );
			connect( _server, SIGNAL( deletedConnection( QString ) ), this, SLOT( removeClient( QString ) ) );
			connect( _server, SIGNAL( deletedConnection( EPOServerClient* ) ),
			         _model, SLOT( removeClient( EPOServerClient* ) ) );
			connect( this, SIGNAL( sendMessage( QString, QVariant ) ), _server, SLOT( sendCommand( QString, QVariant ) ) );

			treeView->setModel( _model );
			treeView->setAlternatingRowColors( true );
			treeView->setItemDelegate( new NSClientModel::ClientDelegate( treeView ) );
			clientview->setModel( _model );
			clientview->setAlternatingRowColors( true );
			clientview->setSelectionModel( treeView->selectionModel() );

		}
		~Clientview() {}
	public slots:
		void newClient( QString n ) {
			qDebug( "Clientview::newClient( QString %s )", n.toStdString().c_str() );
			clientList->addItem( n );
		}
		void removeClient( QString n ) {
			foreach( QListWidgetItem* item, clientList->findItems( n, Qt::MatchExactly ) ) {
				delete item;
			}
			if ( ! clientList->currentItem() ) {
				btnViewOsci->setEnabled( false );
				btnViewFilter->setEnabled( false );
				btnQuitClient->setEnabled( false );
			}
		}
	signals:
		void sendMessage( QString, QVariant );
	private slots:
		void itemSelectionChanged() {
			btnViewOsci->setEnabled( true );
			btnViewFilter->setEnabled( true );
			btnQuitClient->setEnabled( true );
		}
		void btnQuitClientClicked() {
			if ( clientList->currentItem() ) {
				qDebug( "Clientview::btnQuitClientClicked()\n" );
				QString message = clientList->currentItem()->text() + "/quit";
				emit sendMessage( message, QVariant() );
			}
		}
		void btnViewOsciClicked() {
			if ( clientList->currentItem() ) {
				EPOServerClient* client = _server->client( clientList->currentItem()->text() );
				Mainwidget* w = new Mainwidget( client, this );
				w->show();
				if ( !connect( client, SIGNAL( sDisconnected() ), w, SLOT( close() ) ) )
					qDebug( "\n Connection failed!\n" );
			}
		}
		void btnViewFilterClicked() {
			if ( clientList->currentItem() ) {
				EPOServerClient* client = _server->client( clientList->currentItem()->text() );
				client->sendData( "/filter/status", QVariant() );
			}
		}
	private:
		EPOServer* _server;
		ClientModel* _model;
//		Converter* _conv;
};

#endif // MAIN_H

