/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include "myosciwidget.h"
#include "myosciwidget.moc"

#include "osciwidget.h"
//#include "acqiris.h"
#include "acqiristabs_v2.h"
//#include "verticalparameters.h"
//#include "voltageparameters.h"
#include "led.h"
#include "oscitabs.h"
#include "configfile.h"
#include "kcolorcombo.h"

#include "net_server.h"
#include "inttofloat.h"
#include "converter.h"
#include "akhline.h"

#include <Qt/qlayout.h>
#include <Qt/qpushbutton.h>
#include <Qt/qspinbox.h>
#include <Qt/qlineedit.h>
#include <Qt/qcombobox.h>
#include <Qt/qtabwidget.h>
#include <Qt/qcheckbox.h>
#include <Qt/qbuttongroup.h>
#include <Qt/qsplitter.h>

using namespace NSCore;

Mainwidget::Mainwidget( EPOServerClient* socket, QWidget* p )
	: QMainWindow( p )
	, _splitter( new QSplitter( Qt::Horizontal, this ) )
//	, _acqiris( new Acqiris::Acqiris( this ) )
	, _acqirisbox2_widget( new QWidget( _splitter ) )
	, _osciwidget( new OsciWidget( _splitter ) )
	, _oscitabs_widget( new QWidget( _splitter ) )
	, _socket( socket )
{
	_oscitabs = new Ui::OsciTabs();
	_oscitabs->setupUi( _oscitabs_widget );
	_acqirisbox2 = new Ui::AcqirisTabsV2();
	_acqirisbox2->setupUi( _acqirisbox2_widget );

	setCentralWidget( _splitter );
	_splitter->setStretchFactor( 0, 0 );
	_splitter->setStretchFactor( 1, 100 );
	_splitter->setStretchFactor( 2, 0 );

	qDebug( "Mainwidget::Mainwidget( QWidget* %p )", p );
//X 	if ( _acqiris->digitizer( 0 ) ) {
//X 		if ( _acqiris->digitizer( 0 )->channel( 0 ) )
//X 			connect( _acqiris->digitizer( 0 )->channel( 0 ), SIGNAL( gotData( ::Acqiris::ChannelData ) ),
//X 			         _osciwidget, SLOT( getData( ::Acqiris::ChannelData ) ) );
//X 		if ( _acqiris->digitizer( 0 )->channel( 1 ) )
//X 			connect( _acqiris->digitizer( 0 )->channel( 1 ), SIGNAL( gotData( ::Acqiris::ChannelData ) ),
//X 			         _osciwidget, SLOT( getData( ::Acqiris::ChannelData ) ) );
//X 	}

	Converter* conv = new Converter( this );
	connect( _socket->path( "/data" ), SIGNAL( gotMessage( QVariant ) ), conv, SLOT( newData( QVariant ) ) );
	connect( conv, SIGNAL( gotData( ::Acqiris::ChannelData ) ), _osciwidget, SLOT( getData( ::Acqiris::ChannelData ) ) );
	// DisplayBox
	connect( _oscitabs->spinboxNumberOfFades, SIGNAL( valueChanged( int ) ), _osciwidget, SLOT( numberOfFades( int ) ) );
	_oscitabs->checkboxConnectPoints->setChecked( _osciwidget->connectPoints() );
	connect( _oscitabs->checkboxConnectPoints, SIGNAL( toggled( bool ) ), _osciwidget, SLOT( connectPoints( bool ) ) );
	connect( _oscitabs->scalinggroup, SIGNAL( clicked( int ) ), _osciwidget, SLOT( autoScaleProp( int ) ) );
	{
		_oscitabs->spinboxUpperRange->setValue( int( _osciwidget->getMax()*1000 ) );
		IntToFloat *tmp = new IntToFloat( 1E-3, this );
		connect( _oscitabs->spinboxUpperRange, SIGNAL( valueChanged( int ) ), tmp, SLOT( valueChanged( int ) ) );
		connect( tmp, SIGNAL( valueChanged( float ) ), _osciwidget, SLOT( setMax( float ) ) );
		_oscitabs->spinboxLowerRange->setValue( int( _osciwidget->getMin()*1000 ) );
		tmp = new IntToFloat( 1E-3, this );
		connect( _oscitabs->spinboxLowerRange, SIGNAL( valueChanged( int ) ), tmp, SLOT( valueChanged( int ) ) );
		connect( tmp, SIGNAL( valueChanged( float ) ), _osciwidget, SLOT( setMin( float ) ) );
	}
	_oscitabs->colorcombo1->setColor( _osciwidget->color1() );
	connect( _oscitabs->colorcombo1, SIGNAL( activated( const QColor& ) ), _osciwidget, SLOT( setColor1( const QColor& ) ) );
	_oscitabs->colorcombo2->setColor( _osciwidget->color2() );
	connect( _oscitabs->colorcombo2, SIGNAL( activated( const QColor& ) ), _osciwidget, SLOT( setColor2( const QColor& ) ) );
	connect( _oscitabs->btnReset1, SIGNAL( clicked() ), _osciwidget, SLOT( clear1() ) );
	connect( _oscitabs->btnReset2, SIGNAL( clicked() ), _osciwidget, SLOT( clear2() ) );

	// AcqirisBox

//X 	// buttons
//X 	connect( _acqirisbox2->btnCalibrate, SIGNAL( clicked() ), _acqiris->digitizer( 0 ), SLOT( calibrate() ) );
	connect( _acqirisbox2->btnCalibrate, SIGNAL( clicked() ), _socket->path( "/digitizer/calibrate" ), SLOT( sendMessage() ) );
//X 	connect( _acqirisbox2->btnStart, SIGNAL( clicked() ), _acqiris->digitizer( 0 ), SLOT( startAcqisition() ) );
	connect( _acqirisbox2->btnStart, SIGNAL( clicked() ), _socket->path( "/digitizer/start" ), SLOT( sendMessage() ) );
//X 	connect( _acqirisbox2->btnSingleRun, SIGNAL( toggled( bool ) ), _acqiris->digitizer( 0 ), SLOT( singleShot( bool ) ) );
	connect( _acqirisbox2->btnSingleRun, SIGNAL( toggled( bool ) ), _socket->path( "/digitizer/singleshot" ), SLOT( sendMessage( bool ) ) );
//X 	connect( _acqirisbox2->btnStop, SIGNAL( clicked() ), _acqiris->digitizer( 0 ), SLOT( stopAcqisition() ) );
	connect( _acqirisbox2->btnStop, SIGNAL( clicked() ), _socket->path( "/digitizer/stop" ), SLOT( sendMessage() ) );
//X 
//X 	// general parameters
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->timeout->setValue( _acqiris->digitizer( 0 )->acquisitionTimeOut() );
//X 	connect( _acqirisbox2->timeout, SIGNAL( valueChanged( int ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( acquisitionTimeOut( int ) ) );
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->checkboxForceTrigger->setChecked( _acqiris->digitizer( 0 )->forceTrigger() );
//X 	connect( _acqirisbox2->checkboxForceTrigger, SIGNAL( toggled( bool ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( forceTrigger( bool ) ) );
//X 	// samplingrate
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->samplingrate->setText( QString::number( _acqiris->digitizer( 0 )->samplinginterval() ) );
//X 	connect( _acqirisbox2->samplingrate, SIGNAL( textChanged( const QString & ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( setSamplingInterval( const QString & ) ) );
//X 
//X 	// number of samples
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->numberofSamples->setValue( _acqiris->digitizer( 0 )->samples() );
//X 	connect( _acqirisbox2->numberofSamples, SIGNAL( valueChanged( int ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( setSamples( int ) ) );
//X 
//X 	// Trigger Delay
//X 	IntToFloat* triggerDelayCalc = new IntToFloat( 1E-9, this );
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->spinboxTriggerDelay->setValue( int( _acqiris->digitizer( 0 )->delaytime() * 1E9 ) );
//X 	connect( _acqirisbox2->spinboxTriggerDelay, SIGNAL( valueChanged( int ) ),
//X 	         triggerDelayCalc, SLOT( valueChanged( int ) ) );
//X 	connect( triggerDelayCalc, SIGNAL( valueChanged( float ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( delaytime( float ) ) );
//X 
//X 	// Trigger Source
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->comboboxTriggerSource->addItems( _acqiris->digitizer( 0 )->triggersources() );
//X 	connect( _acqirisbox2->comboboxTriggerSource, SIGNAL( activated( const QString & ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( triggersource( const QString & ) ) );
//X 
//X 	// Trigger levels
//X 	IntToFloat* triggerLevelInternCalc = new IntToFloat( 1, this );
//X 	connect( _acqirisbox2->spinboxTriggerLevelInput, SIGNAL( valueChanged( int ) ),
//X 	         triggerLevelInternCalc, SLOT( valueChanged( int ) ) );
//X 	connect( triggerLevelInternCalc, SIGNAL( valueChanged( float ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( triggerlevel( float ) ) );
//X 	IntToFloat* triggerLevelExternCalc = new IntToFloat( 1E-3, this );
//X 	connect( _acqirisbox2->spinboxTriggerLevelExtern, SIGNAL( valueChanged( int ) ),
//X 	         triggerLevelExternCalc, SLOT( valueChanged( int ) ) );
//X 	connect( triggerLevelExternCalc, SIGNAL( valueChanged( float ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( triggerlevel( float ) ) );
//X 	// Trigger slope
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->comboboxTriggerSlope->addItems( _acqiris->digitizer( 0 )->triggerslopes() );
//X 	connect( _acqirisbox2->comboboxTriggerSlope, SIGNAL( activated( const QString& ) ),
//X 	         _acqiris->digitizer( 0 ),SLOT( triggerslope( const QString& ) ) );
//X 
//X 	// Trigger coupling
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		_acqirisbox2->comboboxTriggerCoupling->addItems( _acqiris->digitizer( 0 )->triggercouplings() );
//X 	connect( _acqirisbox2->comboboxTriggerCoupling, SIGNAL( activated( const QString& ) ),
//X 	         _acqiris->digitizer( 0 ), SLOT( triggercoupling( const QString& ) ) );
//X 
//X 	// State-LED
//X 	_acqirisbox2->ledState->setOff();
//X 	connect( _acqiris->digitizer( 0 ), SIGNAL( acquisitionStarted() ), _acqirisbox2->ledState, SLOT( setOn() ) );
//X 	connect( _acqiris->digitizer( 0 ), SIGNAL( acquisitionStopped() ), _acqirisbox2->ledState, SLOT( setOff() ) );
//X 	connect( _acqiris->digitizer( 0 ), SIGNAL( acquisitionStoppedTimeout() ), _acqirisbox2->ledState, SLOT( setError() ) );
//X 
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		if ( _acqiris->digitizer( 0 )->channels() > 0 ) {
//X 			QWidget *tmp = 0;
//X 			// Search for the tab "Channels"
//X 			for ( int j=0; j<_acqirisbox2->tab->count(); j++ )
//X 				if ( _acqirisbox2->tab->tabText( j ) == "Channels" )
//X 					tmp = _acqirisbox2->tab->widget( j );
//X 			if ( tmp != 0 ) {
//X 				QBoxLayout *layout = new QVBoxLayout( tmp );
//X 				for ( int i=0; i<_acqiris->digitizer( 0 )->channels(); ++i ) {
//X 					QWidget *vp_widget = new QWidget( tmp );
//X 					Ui::VerticalParameters *vp = new Ui::VerticalParameters();
//X 					vp->setupUi( vp_widget );
//X 					Ui::VoltageParameters *volt = new Ui::VoltageParameters();
//X 					volt->setupUi( vp->vp );
//X 					volt->fullscale->addItems( _acqiris->digitizer( 0 )->channel( i )->fullscales() );
//X 					connect( volt->fullscale, SIGNAL( activated( const QString& ) ),
//X 							_acqiris->digitizer( 0 )->channel( i ), SLOT( fullscale( const QString& ) ) );
//X 					vp->coupling->addItems( _acqiris->digitizer( 0 )->channel( i )->couplings() );
//X 					connect( vp->coupling, SIGNAL( activated( const QString& ) ),
//X 							_acqiris->digitizer( 0 )->channel( i ), SLOT( coupling( const QString& ) ) );
//X 					vp->bandwidth->addItems( _acqiris->digitizer( 0 )->channel( i )->bandwidths() );
//X 					connect( vp->bandwidth, SIGNAL( activated( const QString& ) ),
//X 							_acqiris->digitizer( 0 )->channel( i ), SLOT( bandwidth( const QString& ) ) );
//X 					IntToFloat* offsetCalc = new IntToFloat( 1E-3, this );
//X 					connect( volt->offset, SIGNAL( valueChanged( int ) ), offsetCalc, SLOT( valueChanged( int ) ) );
//X 					connect( offsetCalc, SIGNAL( valueChanged( float ) ), _acqiris->digitizer( 0 )->channel( i ), SLOT( offset( float ) ) );
//X 					layout->addWidget( new QLabel( QString( "<b>Channel %1</b>" ).arg( i+1 ), tmp ) );
//X 					QCheckBox *cb = new QCheckBox( "Enable this channel", tmp );
//X 					cb->setChecked( _acqiris->digitizer( 0 )->channel( i )->enabled() );
//X 					connect( cb, SIGNAL( toggled( bool ) ), _acqiris->digitizer( 0 )->channel( i ), SLOT( enable( bool ) ) );
//X 					layout->addWidget( cb );
//X 					layout->addWidget( vp_widget );
//X 					if ( i<_acqiris->digitizer( 0 )->channels()-1 )
//X 						layout->addWidget( new AKHLine( tmp ) );
//X 				}
//X 				// Just a simple spacer...
//X 				QWidget *tmp2 = new QWidget( tmp );
//X 				tmp2->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
//X 				layout->addWidget( tmp2 );
//X 			}
//X 		}
//X 
//X 	if ( _acqiris->digitizer( 0 ) )
//X 		if ( _acqiris->digitizer( 0 )->externaltriggers() > 0 ) {
//X 			QWidget *tmp=0;
//X 			// Search for the tab "Ext. Trigger"
//X 			for ( int j=0; j<_acqirisbox2->tab->count(); j++ )
//X 				if ( _acqirisbox2->tab->tabText( j ) == "Ext. Trigger" )
//X 					tmp = _acqirisbox2->tab->widget( j );
//X 			if ( tmp != 0 ) {
//X 				QBoxLayout *layout = new QVBoxLayout( tmp );
//X 				for ( int i=0; i<_acqiris->digitizer( 0 )->externaltriggers(); i+=2 ) {
//X 					QWidget *vp_widget = new QWidget( tmp );
//X 					Ui::VoltageParameters *vp = new Ui::VoltageParameters();
//X 					vp->setupUi( vp_widget );
//X 					vp->fullscale->addItems( _acqiris->digitizer( 0 )->externaltrigger( i )->fullscales() );
//X 					connect( vp->fullscale, SIGNAL( activated( const QString& ) ),
//X 						_acqiris->digitizer( 0 )->externaltrigger( i ), SLOT( fullscale( const QString& ) ) );
//X 					IntToFloat* offsetCalc = new IntToFloat( 1E-3, this );
//X 					connect( vp->offset, SIGNAL( valueChanged( int ) ),
//X 						offsetCalc, SLOT( valueChanged( int ) ) );
//X 					connect( offsetCalc, SIGNAL( valueChanged( float ) ),
//X 						_acqiris->digitizer( 0 )->externaltrigger( i ), SLOT( offset( float ) ) );
//X 					layout->addWidget( new QLabel( QString( "<b>Ext. Trigger %1</b>" ).arg( i+1 ), tmp ) );
//X 					layout->addWidget( vp_widget );
//X 					if ( i<_acqiris->digitizer( 0 )->externaltriggers()-2 )
//X 						layout->addWidget( new AKHLine( tmp ) );
//X 				}
//X 				// Just a simple spacer...
//X 				QWidget *tmp2 = new QWidget( tmp );
//X 				tmp2->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
//X 				layout->addWidget( tmp2 );
//X 			}
//X 		}
//X 
//X #if 0
//X 	// Front panel IO's
//X 	_acqirisbox2->cmbIOA->insertStringList( _acqiris->digitizer( 0 )->controlIO()->inputStates() );
//X 	connect( _acqirisbox2->cmbIOA, SIGNAL( activated( const QString& ) ),
//X 	         _acqiris->digitizer( 0 )->controlIO(), SLOT( setA( const QString& ) ) );
//X 	_acqirisbox2->cmbIOB->insertStringList( _acqiris->digitizer( 0 )->controlIO()->inputStates() );
//X 	connect( _acqirisbox2->cmbIOB, SIGNAL( activated( const QString& ) ),
//X 	         _acqiris->digitizer( 0 )->controlIO(), SLOT( setB( const QString& ) ) );
//X 	// LED
//X 	_acqirisbox2->cmbLEDColor->insertStringList( _acqiris->digitizer( 0 )->led( 0 )->getColors() );
//X 	connect( _acqirisbox2->cmbLEDColor, SIGNAL( activated( const QString& ) ),
//X 	         _acqiris->digitizer( 0 )->led( 0 ), SLOT( setColor( const QString& ) ) );
//X #endif
}
Mainwidget::~Mainwidget() {
	qDebug( "Mainwidget::~Mainwidget()" );
}

void Mainwidget::closeEvent( QCloseEvent* ) {
	delete this;
}

void Mainwidget::saveSettings() {
/*	qDebug( "Mainwidget::saveSettings()" );
	Configfile* config = new Configfile( this );
	
	config->setOption( "display/numberoffades", _oscitabs->spinboxNumberOfFades->value() );
	config->setOption( "display/connectpoints", _oscitabs->checkboxConnectPoints->isChecked() );
	config->setOption( "display/scalinggroup", _oscitabs->scalinggroup->selectedId() );
	config->setOption( "display/upperrange", _oscitabs->spinboxUpperRange->value() );
	config->setOption( "display/lowerrange", _oscitabs->spinboxLowerRange->value() );
	config->setOption( "display/color1", _oscitabs->colorcombo1->color() );
	config->setOption( "display/color2", _oscitabs->colorcombo2->color() );

	config->setOption( "digitizer/timeout", _acqirisbox2->timeout->value() );
	config->setOption( "digitizer/forcetrigger", _acqirisbox2->checkboxForceTrigger->isChecked() );
	config->setOption( "digitizer/samplingrate", _acqirisbox2->samplingrate->text() );

	config->saveConfig( ".test.conf" );
	delete config;*/
}
void Mainwidget::loadSettings() {
/*	qDebug( "Mainwidget::loadSettings()" );
	Configfile* config = new Configfile( ".test.conf", this );

	_oscitabs->spinboxNumberOfFades->setValue( config->getOption( "display/numberoffades" ).toInt() );
	_oscitabs->checkboxConnectPoints->setChecked( config->getOption( "display/connectpoints" ).toBool() );
	_oscitabs->scalinggroup->setButton( config->getOption( "display/scalinggroup" ).toInt() );
	_oscitabs->spinboxUpperRange->setValue( config->getOption( "display/upperrange" ).toInt() );
	_oscitabs->spinboxLowerRange->setValue( config->getOption( "display/lowerrange" ).toInt() );
	_oscitabs->colorcombo1->setColor( config->getOption( "display/color1" ).toColor() );
	_oscitabs->colorcombo2->setColor( config->getOption( "display/color2" ).toColor() );

	_acqirisbox2->timeout->setValue( config->getOption( "digitizer/timeout" ).toInt() );
	_acqirisbox2->checkboxForceTrigger->setChecked( config->getOption( "digitizer/forcetrigger" ).toBool() );
	_acqirisbox2->samplingrate->setText( config->getOption( "digitizer/samplingrate" ).toString() );

	delete config;*/
}

