/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include <QtCore/QDebug>
#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <stdlib.h>
#include <math.h>

// Function to fit is A*exp(- (x-B)**2 / (2* C**2) )
double function( double A, double B, double C, double x ) {
	return A * exp( - pow( x - C,2 ) / ( 2 * pow( C,2 ) ) );
};

double random( double min, double max ) {
	return rand() * ( max-min ) / RAND_MAX + min;
}

struct ParameterSet {
	double A, B, C, quadsum;

	double& operator[]( int n ) {
		if ( n==0 ) return A;
		if ( n==1 ) return B;
		if ( n==2 ) return C;
		/*if ( n==3 )*/ return quadsum;
	}

	double y( double x ) {
		return function( A,B,C, x );
	}
};

bool operator<( const ParameterSet& one, const ParameterSet& two ) {
	return ( one.quadsum < two.quadsum );
};

int main( int argc, char** argv )
{
	qDebug() << "Starting...";

	QFile file( "test.csv" );
	if ( ! file.exists() )
		qFatal( "File test.csv not found!" );

	if ( ! file.open( QIODevice::ReadOnly ) )
		qFatal( "Couldn't open the existing file test.csv!" );

	QTextStream stream( &file );

	QMap<double,double> data;

	while ( ! stream.atEnd() ) {
		double x, y;
		stream >> x >> y;
		data[ x ] = y;
	}
	data.remove( 0.0 );

//	foreach( double x, data.keys() )
//		qDebug() << "x:" << x << "y:" << data[ x ];

	file.close();

	// File is read and closed again...

	srand( 5435 );
	int parametercount = 100;
	std::list < ParameterSet > parameters( parametercount );

	double range_lower[ 3 ], range_upper[ 3 ];
	range_lower[ 0 ] = 500; range_upper[ 0 ] = 600;
	range_lower[ 1 ] = -1; range_upper[ 1 ] = -0.8;
	range_lower[ 2 ] = 0.3; range_upper[ 2 ] = 0.5;

	for ( std::list<ParameterSet>::iterator it = parameters.begin(); it!=parameters.end(); ++it ) {
		for ( int j=0; j<3; ++j )
			( *it )[ j ] = random( range_lower[ j ], range_upper[ j ] );
		( *it )[ 3 ] = 0;
	}
	for ( std::list<ParameterSet>::iterator it = parameters.begin(); it!=parameters.end(); ++it )
		qDebug() << ":" << ( *it )[ 0 ] << ( *it )[ 1 ] << ( *it )[ 2 ] << ( *it )[ 3 ];

	// Fill all parameters with random ones
	for ( std::list<ParameterSet>::iterator it = parameters.begin(); it!=parameters.end(); ++it ) {
		double quadsum = 0;
		foreach ( double x, data.keys() ) {
			quadsum += pow( data[ x ] - function( ( *it )[ 0 ], ( *it )[ 1 ], ( *it )[ 2 ], x ) , 2 );
		}
		( *it )[ 3 ] = quadsum;
	}
	parameters.sort();

	int c=0;
	int runs = 1000;
	while ( c<runs ) {
		++c;
		//qDebug() << "Run" << c;
		double mins[ 3 ], maxs[ 3 ];
		std::list<ParameterSet>::iterator it = parameters.begin();
		for ( int i=0; i<3; ++i )
			mins[ i ] = maxs[ i ] = 0; //( *it )[ i ];
		//qDebug() << "Mins:" << mins[ 0 ] << mins[ 1 ] << mins[ 2 ];
		//qDebug() << "Maxs:" << maxs[ 0 ] << maxs[ 1 ] << maxs[ 2 ];
		for ( int i=0; i<runs/3; ++i ) {
			//qDebug() << i << ":" << ( *it )[ 0 ] << ( *it )[ 1 ] << ( *it )[ 2 ];
			for ( int j=0; j<3; ++j ) {
				if ( ( *it )[ j ] < mins[ j ] ) mins[ j ] = ( *it )[ j ];
				if ( ( *it )[ j ] > maxs[ j ] ) maxs[ j ] = ( *it )[ j ];
			}
			//qDebug() << "    Mins:" << mins[ 0 ] << mins[ 1 ] << mins[ 2 ];
			//qDebug() << "    Maxs:" << maxs[ 0 ] << maxs[ 1 ] << maxs[ 2 ];
			++it;
		}
		//qDebug() << "Mins:" << mins[ 0 ] << mins[ 1 ] << mins[ 2 ];
		//qDebug() << "Maxs:" << maxs[ 0 ] << maxs[ 1 ] << maxs[ 2 ];
		for ( int j=0; j<runs/3 && it != parameters.end(); ++it ) {
			++j;
			for ( int i=0; i<3; ++i ) {
				/// Range aus min/max
				( *it )[ i ] = random( mins[ i ], maxs[ i ] );
			}
			double quadsum = 0;
			foreach ( double x, data.keys() )
				quadsum += pow( data[ x ] - ( *it ).y( x ), 2 );
			( *it )[ 3 ] = quadsum;
		}
		for ( ; it != parameters.end(); ++it ) {
			for ( int i=0; i<3; ++i ) {
				/// Range aus global Range
				( *it )[ i ] = random( range_lower[ i ], range_upper[ i ] );
			}
			double quadsum = 0;
			foreach ( double x, data.keys() )
				quadsum += pow( data[ x ] - ( *it ).y( x ), 2 );
			( *it )[ 3 ] = quadsum;
		}
		parameters.sort();
	}

	int i=0;
	for ( std::list<ParameterSet>::iterator it = parameters.begin(); it!=parameters.end() && i<10; ++it ) {
		++i;
		qDebug() << i << ":" << ( *it )[ 0 ] << ( *it )[ 1 ] << ( *it )[ 2 ] << ( *it )[ 3 ];
	}

	qDebug() << "...finished.";
	return 0;
}

