/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include "reclassifymodel.h"
#include "reclassifymodel.moc"

#include <QtCore/QDebug>
#include <QtCore/QFile>
#include <math.h>

#define PREC 1E-12

ReclassifyModel::ReclassifyModel( QObject* p ) : QAbstractTableModel( p ) {
	qDebug( "ReclassifyModel::ReclassifyModel( QObject* %p )", p );

//	if ( QFile::exists( "ttim-2005-08-31-2.csv" ) )
//		loadFile( "ttim-2005-08-31-2.csv" );
//	if ( QFile::exists( "ttim-2005-09-01-2.csv" ) )
//		loadFile( "ttim-2005-09-01-2.csv" );
}
ReclassifyModel::~ReclassifyModel() {
	qDebug( "ReclassifyModel::~ReclassifyModel()" );
}

int ReclassifyModel::rowCount( const QModelIndex& ) const {
	return _classtable.size();
}
int ReclassifyModel::columnCount( const QModelIndex& ) const {
	return 2;
}

QVariant ReclassifyModel::data( const QModelIndex& index, int role ) const {
	//qDebug( "ReclassifyModel::data( const QModelIndex&, int %i )", role );
	//qDebug() << index;
	if ( ! index.isValid() )
		return QVariant();
	if ( role == Qt::DisplayRole ) {
		if ( index.column() == 0 )
			return _classtable.keys()[ index.row() ];
		else
			return _classtable.values()[ index.row() ];
	}
	return QVariant();
}

int ReclassifyModel::classes() {
	return _classtable.size();
}

void ReclassifyModel::setClasses( int n ) {
	//qDebug( "ReclassifyModel::setClasses( int %i )", n );
	reclassify( n );
}

void ReclassifyModel::reclassify( int n ) {
	double mintime = 1000;
	double maxtime = -1000;
	foreach ( double item, _table.keys() ) {
		if ( item > maxtime ) maxtime = item;
		if ( item < mintime ) mintime = item;
	}
	//qDebug( "min = %e, max = %e", mintime, maxtime );

	double interval = maxtime - mintime;
	double step = interval / n;
	step = step - remainder( step, PREC / 100 );

	beginRemoveRows( QModelIndex(), 0, _classtable.size() );
	_classtable.clear();
	endRemoveRows();

	beginInsertRows( QModelIndex(), 0, n );
	for ( int i=0; i<n; ++i ) _classtable[ i*step ] = 0;
	QList<double> _keys = _table.keys();
	QList<double> _values = _table.values();
	for ( int i=0; i<_table.size(); ++i ) {
		int j=0;
		while ( _keys[ i ] > j*step ) { ++j; }
		_classtable[ j*step ] += _values[ i ];
	}
	//qDebug( "After classification is _classtable.size() = %i", _classtable.size() );
	endInsertRows();

	emit dataChanged( createIndex( 0,0 ), createIndex( _classtable.size(), 1 ) );
}

void ReclassifyModel::loadFile( QString name ) {
	//qDebug() << "ReclassifyModel::loadFile( " << name << " )";
	QFile file( this );
	if ( QFile::exists( name ) ) {
		file.setFileName( name );
		if ( file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
			QTextStream stream( &file );
			QString line = stream.readLine();
			int i=0;
			while ( ! line.isNull() ) {
				line = stream.readLine();
				if ( line.count( " " ) > 0 ) {
					double time = line.section( " ", 0,0 ).toDouble();
					int count = line.section( " ", 1,1 ).toInt();
					_table[ fabs( time ) - remainder( fabs( time ), PREC ) ] += count;
				}
				++i;
			}
			file.close();
			reclassify( _table.count() );
		} else
			qWarning( "Couldn't open the file" );
	} else
		qWarning( "File %s does not exist!", name.toStdString().c_str() );
}

void ReclassifyModel::clear() {
	beginRemoveRows( QModelIndex(), 0, _classtable.size() );
	_classtable.clear();
	endRemoveRows();
}

void ReclassifyModel::saveFile( QString path ) {
	QFile *file = new QFile( path );
	if ( file->open( QIODevice::WriteOnly ) ) {
		QTextStream stream( file );
		if ( path.endsWith( ".dat" ) ) {
			for ( int i=0; i<classes(); ++i )
				stream << _classtable[ _classtable.keys()[ i ] ] << endl;
		} else {
			for ( int i=0; i<_classtable.size(); ++i )
				stream << _classtable.key( i ) << " " << _classtable[ _classtable.keys()[ i ] ] << endl;
		}
		file->close();
	}
}

