/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */


#include "main.h"
#include "main.moc"

#include <iostream>
#include <stdio.h>
#include <math.h>
#include <acqiris.h>
#include <QtGui/QPainter>
#include <QtGui/QKeyEvent>
#include <QtGui/QApplication>
#include <QtCore/QTimer>

TTIM::TTIM( QWidget* p ) : QWidget( p ), _max( 0 ) {
	_acqiris = new Acqiris::Acqiris( this );
	for ( int i=0; i<_acqiris->numberOfDigitizers() && i<1; ++i ) {
		_digitizer = _acqiris->digitizer( i );
		qDebug( "setting sample rate to %e", 1/4E9 );
		_digitizer->setSamplingInterval( 1/4E9 );
		_digitizer->channel( 0 )->enable( true );
		_digitizer->channel( 0 )->fullscale( "1V" );
		connect( _digitizer->channel( 0 ), SIGNAL( gotData( ::Acqiris::ChannelData ) ), this, SLOT( gotData( ::Acqiris::ChannelData ) ) );
		_digitizer->triggerslope( "Negative" );
		_digitizer->triggerlevel( -10 );
		_digitizer->delaytime( -10E-9 );
		_digitizer->forceTrigger( true );
		_digitizer->setSamples( 100 );
		_digitizer->acquisitionTimeOut( 10000 );
	}
}
TTIM::~TTIM() {
	qDebug( "TTIM::~TTIM()" );
	saveData();
}

void TTIM::gotData( ::Acqiris::ChannelData d ) {
	static int c=0;
	c++;
	double hortime = d.horizontalposition;
	hortime = fabs( hortime ) - remainder( fabs( hortime ), 1E-13 ); // Rundet auf gewünschte Stellenanzahl
	_ttis[ hortime ] += 1;
	if ( _ttis[ hortime ] > _max ) _max = _ttis[ hortime ];
	if ( c%20 == 0 ) {
		std::cerr << ".";
		repaint();
		c = 0;
	}
}

void TTIM::keyPressEvent( QKeyEvent* e ) {
	static bool running = false;
	if ( e->key() == Qt::Key_Space ) {
		if ( !running ) {
			_digitizer->startAcqisition();
			running=true;
			QTimer::singleShot( 1000*60*10, this, SLOT( saveData() ) );
		} else {
			_digitizer->stopAcqisition();
			running=false;
			setMinimumWidth( _ttis.size() );
		}
	}
}

void TTIM::paintEvent( QPaintEvent* ) {
//	std::cerr << "p";
	QPainter p( this );
	p.setPen( QColor( 0,0,0 ) );
	for ( QMap<double,int>::Iterator it=_ttis.begin(); it!=_ttis.end(); ++it ) {
		int x = int( fabs( it.key() )/( 1/4E9 ) *width() );
		p.drawLine( x,0, x,int( height()*double( *it ) / _max ) );
	}
	p.setPen( QColor( 0,0,255 ) );
	p.drawText( width()/2, height()/2, QString::number( _ttis.size() ) );
}

void TTIM::saveData() {
	FILE* file = fopen( ".ttim.csv", "w" );
	if ( file ) {
		for ( QMap<double,int>::Iterator it = _ttis.begin(); it!=_ttis.end(); ++it ) {
			fprintf( file, "%e %i\n", it.key(), it.value() );
		}
		fclose( file );
	}
	QTimer::singleShot( 1000*60*10, this, SLOT( saveData() ) );
}


int main( int argc, char** argv ) {
	std::cout << "TTI-M" << std::endl;

	QApplication* a = new QApplication( argc, argv );

	TTIM* ttim = new TTIM( 0 );
	ttim->show();

	std::cout << "Beginning..." << std::endl;
	int ret = a->exec();
	std::cout << "Quit." << std::endl;
	delete ttim;
	return ret;
}

