/*
 * Copyright 2005-2007 EPOS-project
 *                     (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
 */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

/*############################################################################*/

void _load_chn_in ( void );
void _save_chn_out ( void );
void _mod_chn ( void );
void _export_chn_out( void );

/*############################################################################*/

struct spec_chn
{
	char	name[60];
	short	res_1;
	short	res_2;
	short	res_3;
	char	start_time_sec[2];
	long	real_time;
	long	live_time;
	char	start_date[8];
	char	start_time[4];
	short	channel_offset;
	short	channel_number;
	long 	data[35000]; /*set to max 35 k */
	short	flag;
	short	res_4;
	float	ecal_offset;
	float	ecal_gain;
	float	ecal_quad;
	float	peakshape_offset;
	float	peakshape_gain;
	float	peakshape_quad;
	char	res_6[228];
	char	res_7;
	char	detector[63];
	char	res_8;
	char	sample[63];
	char	res_9[128];

};

struct spec_chn chn, chn_in, chn_out; 

/*############################################################################*/
/* ORTEC CHN INTEGER FORMAT for gamma spectra .chn 							  */
/*																			  */
/*  byte data	description 												  */
/* 	 0	2		should be -1, 2 byte integer								  */
/* 	 2	2		mca number or detector number, 2 byte integer				  */
/* 	 4	2		segment number, set to 1 in UMCBI, 2 byte integer 			  */
/*   6	2		ASCII seconds of start 										  */
/*   8	4		real time (inc. of 20 ms) 4 byte integer 					  */
/*  12	4		live time (inc. of 20 ms) 4 byte integer 					  */
/*  16	8		start date as ASCII DDMMMYY or binary zeros if unknown, 	  */
/*				the last 1 indicates 2000++ 								  */
/*  24	4		start time as ASCII HHMM or binary zeros if unknown 		  */
/*  28	2		channel offset, 2 byte integer								  */
/*  30	2		number of channels, 2 byte integer							  */
/*  32  4       ...data as 4 byte integer 									  */ 
/* 										   									  */
/*  0	2		should be -101 for older software - GV V2 or Maestro V3 	  */
/*  2	2		reserved 													  */
/*  4	4		energy calibration zero intercept, 0.0 for uncal, 4 byte float*/
/*  8	4		energy calibration slope, 1.0 for uncal, 4 byte float		  */
/*  12	4		reserved (float or integer ?)								  */
/*  16	4		peak shape calib. zero intercept, 1.0 for uncal, 4 byte float */
/*  20	4		peak shape calib. slope, 0.0 for uncal, 4 byte float		  */
/*  24	232		reserved													  */
/* 256	1		length of detector description								  */
/* 257	63		detector description										  */
/* 320	1		length of sample description 								  */
/* 321	63		sample description											  */
/* 384	128		reserved - total length is 512 byte							  */
/* 										   									  */
/*  0	2		should be -102 for new software - GV V2.2 or Maestro V3.1 	  */
/*  2	2		reserved 													  */
/*  4	4		energy calibration zero intercept, 0.0 for uncal, 4 byte float*/
/*  8	4		energy calibration slope, 1.0 for uncal, 4 byte float		  */
/*  12	4		energy calibration quadratic term, 0.0 for uncal, 4 byte float*/
/*  16	4		peak shape calib. zero intercept, 1.0 for uncal, 4 byte float */
/*  20	4		peak shape calib. slope, 0.0 for uncal, 4 byte float	  	  */
/*  24	4		peak shape calib. quadratic term, 0.0 for uncal, 4 byte float */
/*  28	228		reserved													  */
/* 256	1		length of detector description								  */
/* 257	63		detector description										  */
/* 320	1		length of sample description 								  */
/* 321	63		sample description											  */
/* 384	128		reserved - total length is 512 byte							  */
/*############################################################################*/

int main(int argc, char *argv[])
{

	printf ( "SPECTRUM MOD\n\n" );
	printf ( "input chn  : test.chn\n" );
	printf ( "output chn : test_mod\n\n" );

	/*                                                              */
	/*    _load_chn() - load test.chn into struct chn_in            */
	/*                                                              */
	/*    copy struct chn_in to struct chn                          */
	/*                                                              */
	/*    _mod_chn()  - modify the chn struct                       */
	/*                                                              */
	/*    copy struct chn to struct chn_out                         */
	/*                                                              */
	/*    _save_chn() - save struct chn_out to test_mod.chn         */
	/*                                                              */


	_load_chn_in ();

//	chn = chn_in;

//	_mod_chn ();

	chn_out = chn_in;

	_save_chn_out ();

	_export_chn_out();

//  	system("PAUSE");	
  	return 0;
} 

/*############################################################################*/

void _mod_chn ( void )
{
	int i;
	
	for ( i = 0 ; i <= chn.channel_number; i+=1)
		chn.data[i] *= 2;
}

/*############################################################################*/

void _load_chn_in ( void )
{
	FILE *fp;
	int i;

	strcpy ( chn_in.name , "test.chn" );
	fp = fopen( chn_in.name , "rb" );
	if ( fp == NULL )
	{
		printf ( "error : open file : %s\n" , chn_in.name );
  	    system ( "PAUSE" );
		exit ( 1 );
	}

	/* read first info block */
	fseek ( fp, 0 , SEEK_SET );
	fread ( &chn_in.res_1 , 2 , 1 , fp );

	fseek ( fp, 2 , SEEK_SET );
	fread ( &chn_in.res_2 , 2 , 1 , fp );

	fseek ( fp, 4 , SEEK_SET );
	fread ( &chn_in.res_3 , 2 , 1 , fp );

	fseek ( fp, 6 , SEEK_SET );
	fread ( &chn_in.start_time_sec , 2 , 1 , fp );

	fseek ( fp, 8 , SEEK_SET );
	fread ( &chn_in.real_time , 4 , 1 , fp );

	fseek ( fp, 12 , SEEK_SET );
	fread ( &chn_in.live_time , 4 , 1 , fp );

	fseek ( fp, 16 , SEEK_SET );
	fread ( &chn_in.start_date , 7 , 1 , fp );

	fseek ( fp, 24 , SEEK_SET );
	fread ( &chn_in.start_time , 4 , 1 , fp );

	fseek ( fp, 28 , SEEK_SET );
	fread ( &chn_in.channel_offset , 2 , 1 , fp );

	fseek ( fp, 30 , SEEK_SET );
	fread ( &chn_in.channel_number , 2 , 1 , fp );

	/* read data block */

	fseek ( fp, 32 , SEEK_SET );
	fread ( &chn_in.data , 4 , chn_in.channel_number , fp );

	/* read last info block */

	fseek ( fp , -512 , SEEK_END );
	fread ( &chn_in.flag ,	   2 , 1 , fp );

	fseek ( fp , -510 , SEEK_END );
	fread ( &chn_in.res_4 ,	   2 , 1 , fp );

	fseek ( fp , -508 , SEEK_END );
	fread ( &chn_in.ecal_offset ,	     4 , 1 , fp );	

	fseek ( fp , -504 , SEEK_END );
	fread ( &chn_in.ecal_gain ,        4 , 1 , fp );	

	fseek ( fp , -500 , SEEK_END );
	fread ( &chn_in.ecal_quad ,		 4 , 1 , fp );	

	fseek ( fp , -496 , SEEK_END );
	fread ( &chn_in.peakshape_offset , 4 , 1 , fp );	

	fseek ( fp , -492 , SEEK_END );
	fread ( &chn_in.peakshape_gain ,	 4 , 1 , fp );	

	fseek ( fp , -488 , SEEK_END );
	fread ( &chn_in.peakshape_quad ,   4 , 1 , fp );	

	fseek ( fp , -484 , SEEK_END );
	fread ( &chn_in.res_6 ,	   228 , 1 , fp );

	fseek ( fp , -256 , SEEK_END );
	fread ( &chn_in.res_7 ,	   1 , 1 , fp );

	fseek ( fp , -255 , SEEK_END );
	fread ( &chn_in.detector ,	    63 , 1 , fp );	

	fseek ( fp , -192 , SEEK_END );
	fread ( &chn_in.res_8 ,	   1 , 1 , fp );

	fseek ( fp , -191 , SEEK_END );
	fread ( &chn_in.sample ,	   63 , 1 , fp );

	fseek ( fp , -128 , SEEK_END );
	fread ( &chn_in.res_9 ,	   128 , 1 , fp );

	printf ( "name                 : %s\n" , chn_in.name );
	printf ( "start time sec       : %2s\n", chn_in.start_time_sec );
	printf ( "real time            : %7d\n", chn_in.real_time );
	printf ( "live time            : %7d\n", chn_in.live_time );
	printf ( "start date           : %7s\n", chn_in.start_date );
	printf ( "start time           : %4s\n", chn_in.start_time );
	printf ( "channel offset       : %7d\n", chn_in.channel_offset );
	printf ( "number of channels   : %7d\n", chn_in.channel_number );
	printf ( "old new flag         : %7d\n", chn_in.flag );
	printf ( "detektor             : %30s\n", chn_in.detector );
	printf ( "energycal gains      : %7f\n", chn_in.ecal_gain);
	printf ( "energycal offset     : %7f\n", chn_in.ecal_offset);
	printf ( "energycal quad       : %7f\n", chn_in.ecal_quad);
	printf ( "peakshape offset     : %7f\n", chn_in.peakshape_offset);
	printf ( "peakshape gain       : %7f\n", chn_in.peakshape_gain);
	printf ( "peakshape quad       : %7f\n", chn_in.peakshape_quad);

	for (i =1043 ; i < 1050 ; i += 1 )
  		printf ( "sample data          : %7d - %7d\n", i,chn_in.data[i] );

	fclose ( fp );

}

/*############################################################################*/

void _save_chn_out ( void )
{
	FILE *fp;
	long position;
	
	strcpy( chn_out.name , "test_mod.chn" );

	fp = fopen( chn_out.name , "wb" );
	if ( fp == NULL )
	{
		printf ( "error : open file : %s\n" , chn_out.name );
		exit ( 1 );
	}

	/* write first info block */
	fseek ( fp, 0 , SEEK_SET );
	fwrite ( &chn_out.res_1 , 2 , 1 , fp );

	fseek ( fp, 2 , SEEK_SET );
	fwrite ( &chn_out.res_2 , 2 , 1 , fp );

	fseek ( fp, 4 , SEEK_SET );
	fwrite ( &chn_out.res_3 , 2 , 1 , fp );

	fseek ( fp, 6 , SEEK_SET );
	fwrite ( &chn_out.start_time_sec , 2 , 1 , fp );

	fseek ( fp, 8 , SEEK_SET );
	fwrite ( &chn_out.real_time , 4 , 1 , fp );

	fseek ( fp, 12 , SEEK_SET );
	fwrite ( &chn_out.live_time , 4 , 1 , fp );

	fseek ( fp, 16 , SEEK_SET );
	fwrite ( &chn_out.start_date , 7 , 1 , fp );

	fseek ( fp, 24 , SEEK_SET );
	fwrite ( &chn_out.start_time , 4 , 1 , fp );

	fseek ( fp, 28 , SEEK_SET );
	fwrite ( &chn_out.channel_offset , 2 , 1 , fp );

	fseek ( fp, 30 , SEEK_SET );
	fwrite ( &chn_out.channel_number , 2 , 1 , fp );

	/* write data block */

	fseek ( fp, 32 , SEEK_SET );
	fwrite ( &chn_out.data , 4 , chn_out.channel_number , fp );

	/* write last info block */

	if ( ftell (fp) != (long)(chn_out.channel_number * 4 + 32) )
	{
		printf ( "error : actual position in file is inconsistant : %s\n" , chn_out.name );
		printf ( "calculated position : %d\n" , chn_out.channel_number * 4 + 32 );
		printf ( "actual     position : %d\n" , ftell ( fp ) );
  	    system ( "PAUSE" );
		exit ( 1 );
	}

	position = ftell ( fp );

	fseek ( fp , position + 0 , SEEK_SET );
	fwrite ( &chn_out.flag ,	   2 , 1 , fp );

	fseek ( fp , position + 2 , SEEK_SET );
	fwrite ( &chn_out.res_4 ,	   2 , 1 , fp );

	fseek ( fp , position + 4 , SEEK_SET );
	fwrite ( &chn_out.ecal_offset ,	     4 , 1 , fp );	

	fseek ( fp , position + 8 , SEEK_SET );
	fwrite ( &chn_out.ecal_gain ,        4 , 1 , fp );	

	fseek ( fp , position + 12 , SEEK_SET );
	fwrite ( &chn_out.ecal_quad ,		 4 , 1 , fp );	

	fseek ( fp , position + 16 , SEEK_SET );
	fwrite ( &chn_out.peakshape_offset , 4 , 1 , fp );	

	fseek ( fp , position + 20 , SEEK_SET );
	fwrite ( &chn_out.peakshape_gain ,	 4 , 1 , fp );	

	fseek ( fp , position + 24 , SEEK_SET );
	fwrite ( &chn_out.peakshape_quad ,   4 , 1 , fp );	

	fseek ( fp , position + 28 , SEEK_SET );
	fwrite ( &chn_out.res_6 ,	   228 , 1 , fp );

	fseek ( fp , position + 256 , SEEK_SET );
	fwrite ( &chn_out.res_7 ,	   1 , 1 , fp );

	fseek ( fp , position + 257 , SEEK_SET );
	fwrite ( &chn_out.detector ,	    63 , 1 , fp );	

	fseek ( fp , position + 320 , SEEK_SET );
	fwrite ( &chn_out.res_8 ,	   1 , 1 , fp );

	fseek ( fp , position + 321 , SEEK_SET );
	fwrite ( &chn_out.sample ,	   63 , 1 , fp );

	fseek ( fp , position + 384 , SEEK_SET );
	fwrite ( &chn_out.res_9 ,	   128 , 1 , fp );

	fclose ( fp );
}

void _export_chn_out() {
	int i=0;
	for ( i=0; i<chn_out.channel_number; ++i ) {
		printf( "%i : %i\n", i, chn_out.data[ i ] );
		if ( i==50 ) i = chn_out.channel_number - i;
	}
}

/*############################################################################*/

