/* Types */
typedef long Device_Link;
enum Device_AddrFamily {
	DEVICE_TCP,
	DEVICE_UDP
};
typedef long Device_Flags;

/* Error types */
typedef long Device_ErrorCode;
struct Device_Error {
	Device_ErrorCode error;
};

/* Parameters */
struct Create_LinkParms {
	long clientId;
	bool lockDevice;
	unsigned long lock_timeout;
	string device<>;
};
struct Create_LinkResp {
	Device_ErrorCode error;
	Device_Link lid;
	unsigned short abortPort;
	unsigned long maxRecvSize;
};

struct Device_WriteParms {
	Device_Link lid;
	unsigned long io_timeout;
	unsigned long lock_timeout;
	Device_Flags flags;
	opaque data<>;
};
struct Device_WriteResp {
	Device_ErrorCode error;
	unsigned long size;
};

struct Device_ReadParms {
	Device_Link lid;
	unsigned long requestSize;
	unsigned long io_timeout;
	unsigned long lock_timeout;
	Device_Flags flags;
	char termChar;
};
struct Device_ReadResp {
	Device_ErrorCode error;
	long reason;
	opaque data<>;
};

struct Device_ReadStbResp {
	Device_ErrorCode error;
	unsigned char stb;
};
struct Device_GenericParms {
	Device_Link lid;
	Device_Flags flags;
	unsigned long lock_timeout;
	unsigned long io_timeout;
};
struct Device_RemoteFunc {
	unsigned long hostAddr;
	unsigned short hostPort;
	unsigned long progNum;
	unsigned long progVers;
	Device_AddrFamily progFamily;
};
struct Device_EnableSrqParms {
	Device_Link lid;
	bool enable;
	opaque handle<40>;
};
struct Device_LockParms {
	Device_Link lid;
	Device_Flags flags;
	unsigned long lock_timeout;
};

struct Device_DocmdParms {
	Device_Link lid;
	Device_Flags flags;
	unsigned long io_timeout;
	unsigned long lock_timeout;
	long cmd;
	bool network_order;
	long datasize;
	opaque data_in<>;
};
struct Device_DocmdResp {
	Device_ErrorCode error;
	opaque data_out<>;
};

/* Application interface */
program DEVICE_ASYNC {
	version DEVICE_ASYNC_VERSION {
		Device_Error device_abort (Device_Link) = 1;
	} = 1;
} = 0x0607B0;

program DEVICE_CORE {
	version DEVICE_CORE_VERSION {
		Create_LinkResp		create_link			(Create_LinkParms)		= 10;
		Device_WriteResp	device_write		(Device_WriteParms)		= 11;
		Device_ReadResp		device_read			(Device_ReadParms)		= 12;
		Device_ReadStbResp	device_readstb		(Device_GenericParms)	= 13;
		Device_Error		device_trigger		(Device_GenericParms)	= 14;
		Device_Error		device_clear		(Device_GenericParms)	= 15;
		Device_Error		device_remote		(Device_GenericParms)	= 16;
		Device_Error		device_local		(Device_GenericParms)	= 17;
		Device_Error		device_lock			(Device_LockParms)		= 18;
		Device_Error		device_unlock		(Device_Link)			= 19;
		Device_Error		device_enable_srq	(Device_EnableSrqParms)	= 20;
		Device_DocmdResp	device_docmd		(Device_DocmdParms)		= 22;
		Device_Error		destroy_link		(Device_Link)			= 23;
		Device_Error		create_intr_chan	(Device_RemoteFunc)		= 25;
		Device_Error		destroy_intr_chan	(void)					= 26;
	} = 1;
} = 0x0607AF;

